import * as React from 'react';

import Axios from 'axios';

import { Fabric } from 'office-ui-fabric-react/lib/Fabric';

import { CommandBar } from 'office-ui-fabric-react/lib/CommandBar';

import { TextField } from 'office-ui-fabric-react/lib/TextField';

import { Stack } from 'office-ui-fabric-react/lib/Stack';

import { Dropdown, IDropdownOption } from 'office-ui-fabric-react/lib/Dropdown';

import { Title } from '../Title';

import { classNamesFunction } from 'office-ui-fabric-react/lib/Utilities';

import { IDrawerGroupsProps, IDrawerGroupsStyleProps, IDrawerGroupsStyles } from './DrawerGroups.types';

import { IMovableOptionsListElement, MovableOptionsList } from '../MovableOptionsList';

import { MessageBar, MessageBarType } from 'office-ui-fabric-react/lib/MessageBar';

import './DrawerGroups.css';

const getClassNames = classNamesFunction<IDrawerGroupsStyleProps, IDrawerGroupsStyles>();

const delayedTimeout = 10000;

export interface IDrawerGroupsState {
  name: string;
  description: string;
  sites: IDropdownOption[];
  selectedSite?: number;
  drawers: IMovableOptionsListElement[];
  validateFieldsMessage: string | undefined;
}

export class DrawerGroupsBase extends React.PureComponent<IDrawerGroupsProps, IDrawerGroupsState> {
  public static defaultProps = {
    labelCmdBarBtnSave: 'Save',
    labelCmdBarBtnEdit: 'Edit',
    labelCmdBarBtnCancel: 'Cancel',
    labelValidationMessage: 'The following fields are required:',
    labelDrawerGroupsValidationMessage: 'Drawer Groups needs to have at least one Drawer',
    labelFormName: 'Drawer Group Name',
    labelFormSite: 'Site',
    labelFormSitePlaceholder: 'Select a Site',
    labelFormDescription: 'Drawer Group Description',
    labelFormMovableAvailable: 'Site Drawers',
    labelFormMovableSelected: 'Group Drawers',
    labelFormMovableProperty: 'Name'
  };
  private initialState: IDrawerGroupsState = {
    name: '',
    description: '',
    sites: [],
    drawers: [],
    validateFieldsMessage: undefined
  };

  constructor(props: IDrawerGroupsProps) {
    super(props);
    this.state = this.initialState;
  }

  public componentDidMount() {
    this.init();
  }

  public componentDidUpdate(prevProps: IDrawerGroupsProps) {
    if (this.props.id !== prevProps.id) {
      this.init();
    }
  }

  private delayedExecution = (callback: any): void => {
    setTimeout(callback, delayedTimeout);
  };

  private closeErrorsMessage = (): void => {
    this.setState({ validateFieldsMessage: undefined });
  };

  private init() {
    if (this.props.id) {
      this.componentDidMountInEditMode();
    } else {
      this.componentDidMountInNewMode();
    }
  }

  private componentDidMountInNewMode() {
    Axios.get(this.props.baseUrl + '/sites')
      .then(response => {
        const data = response.data.data;
        const options: IDropdownOption[] = [];
        for (let element of data) {
          const option: IDropdownOption = {
            key: element.id,
            text: element.name,
            selected: false
          };
          options.push(option);
        }
        this.setState({ sites: options });
      })
      .catch(error => console.log(error));
  }

  private componentDidMountInEditMode() {
    const { baseUrl, id } = this.props;
    Axios.get(`${baseUrl}/drawergroups/${id}`)
      .then(response => {
        const dto = response.data;

        Axios.get(this.props.baseUrl + '/sites')
          .then(response => {
            const data = response.data.data;
            const options: IDropdownOption[] = [];
            for (let element of data) {
              const option: IDropdownOption = {
                key: element.id,
                text: element.name,
                selected: element.id == dto.site ? true : false
              };
              options.push(option);
            }
            this.setState({ name: dto.name, description: dto.description, sites: options, selectedSite: dto.site, drawers: dto.groups });
          })
          .catch(error => console.log(error));
      })
      .catch(error => console.log(error));
  }

  private loadSiteDrawers(id: any) {
    const url = `${this.props.baseUrl}/sites/${id}/drawers`;
    Axios.get(url)
      .then(response => {
        const data = response.data.data;
        this.setState({ selectedSite: id, drawers: data });
      })
      .catch(error => console.log(error));
  }

  private getFormCommandBarItems = (id: any) => {
    const { labelCmdBarBtnSave, labelCmdBarBtnEdit, labelCmdBarBtnCancel } = this.props;
    const formCommandBarItems = [
      {
        key: 'Save',
        name: labelCmdBarBtnSave,
        iconProps: {
          iconName: 'Save'
        },
        onClick: () => this.onSubmit()
      },
      {
        key: 'Edit',
        name: labelCmdBarBtnEdit,
        iconProps: {
          iconName: 'Edit'
        },
        onClick: () => this.onSubmit()
      },
      {
        key: 'Cancel',
        name: labelCmdBarBtnCancel,
        iconProps: {
          iconName: 'Cancel'
        },
        onClick: () => this.props.onCancel()
      }
    ];

    const removeIdx = id ? 0 : 1;
    formCommandBarItems.splice(removeIdx, 1);

    return formCommandBarItems;
  };

  private onSubmit = (): void => {
    if (!this.validate()) {
      return;
    }

    const { baseUrl, id, onSubmitSuccess, onSubmitError } = this.props;
    const { name, selectedSite, description, drawers } = this.state;

    const drawerGroups = [];

    for (let drawer of drawers) {
      if (drawer.selected) {
        const dgroup = {
          id: drawer.key,
          name: drawer.text
        };
        drawerGroups.push(dgroup);
      }
    }

    const jsonResult = JSON.stringify({
      id: id,
      name: name,
      description: description,
      site: selectedSite,
      groups: drawerGroups
    });

    const method = id ? 'post' : 'put';
    const url = `${baseUrl}/drawergroups`;

    console.log(url);
    console.log('Data Submited to server by', method, jsonResult);

    Axios.request({
      url: url,
      method: method,
      data: jsonResult,
      headers: {
        Accept: 'application/json',
        'Content-Type': 'application/json'
      }
    })
      .then(response => {
        onSubmitSuccess(response.data);
      })
      .catch(error => onSubmitError(error));
  };

  private validate = (): boolean => {
    const { labelDrawerGroupsValidationMessage } = this.props;
    const { name, selectedSite, drawers } = this.state;
    const invalid: string[] = [];

    if (!name) {
      invalid.push('Name');
    }
    if (!selectedSite) {
      invalid.push('Site');
    }

    let selected = 0;
    for (let group of drawers) {
      if (group.selected) {
        selected++;
        break;
      }
    }

    if (selected === 0) {
      const msg = '' + labelDrawerGroupsValidationMessage;
      invalid.push(msg);
    }

    if (invalid.length > 0) {
      const errorsAsString = invalid.map((error: string) => error).join(' | ');
      this.setState({ validateFieldsMessage: `[ ${errorsAsString} ]` });
      this.delayedExecution(this.closeErrorsMessage);
      return false;
    }

    return true;
  };

  public render(): JSX.Element {
    const {
      className,
      id,
      styles,
      theme,
      title,
      labelValidationMessage,
      labelFormName,
      labelFormSite,
      labelFormSitePlaceholder,
      labelFormDescription,
      labelFormMovableAvailable,
      labelFormMovableSelected,
      labelFormMovableProperty
    } = this.props;
    const { description, drawers, name, selectedSite, sites, validateFieldsMessage } = this.state;
    const classNames = getClassNames(styles, { theme: theme!, className });

    return (
      <Fabric className={`ms-Grid ${classNames.root}`}>
        <div className="ms-Grid-row" style={{ marginTop: '1px', marginBottom: '15px' }}>
          <div className="ms-Grid-col ms-sm12">
            <CommandBar items={this.getFormCommandBarItems(id)} />
          </div>
        </div>
        <div className="ms-Grid-row">
          <div className="ms-Grid-col ms-sm12">
            <Title title={title ? title : ''} icon={true} />
          </div>
        </div>
        {validateFieldsMessage && (
          <div className="ms-Grid-row">
            <div className="ms-Grid-col ms-sm12">
              <MessageBar
                messageBarType={MessageBarType.error}
                isMultiline={false}
                onDismiss={this.closeErrorsMessage}
                dismissButtonAriaLabel="Close"
              >
                {labelValidationMessage} {validateFieldsMessage}
              </MessageBar>
            </div>
          </div>
        )}
        <div className="ms-Grid-row">
          <div className="ms-Grid-col ms-sm4">
            <Stack
              horizontalAlign="start"
              tokens={{
                childrenGap: 10
              }}
              padding={10}
            >
              <span>
                <TextField
                  key="DrawerGroupName"
                  value={name}
                  label={labelFormName}
                  required
                  styles={{ fieldGroup: { width: 250 } }}
                  onBlur={e => {
                    this.setState({ name: e.target.value });
                  }}
                />
              </span>
              <span>
                <Dropdown
                  label={labelFormSite}
                  options={sites}
                  required
                  placeholder={labelFormSitePlaceholder}
                  defaultSelectedKey={selectedSite}
                  styles={{ dropdown: { width: 250 } }}
                  onChange={(event, option?: IDropdownOption) => this.loadSiteDrawers(option ? option.key : -1)}
                />
              </span>
              <span>
                <TextField
                  key="DrawerGroupDesc"
                  value={description}
                  label={labelFormDescription}
                  multiline
                  styles={{ fieldGroup: { width: 250 } }}
                  rows={5}
                  onBlur={e => {
                    this.setState({ description: e.target.value });
                  }}
                />
              </span>
            </Stack>
          </div>
          <div className="ms-Grid-col ms-sm8" style={{ marginTop: '10px' }}>
            <MovableOptionsList
              elements={drawers}
              height="400px"
              disableUpAndDown={true}
              availableLabel={labelFormMovableAvailable}
              selectedLabel={labelFormMovableSelected}
              property={labelFormMovableProperty}
            />
          </div>
        </div>
      </Fabric>
    );
  }
}
