import * as React from 'react';

import Axios from 'axios';

import { createRef, RefObject, classNamesFunction } from 'office-ui-fabric-react/lib/Utilities';

import { SearchBox } from 'office-ui-fabric-react/lib/SearchBox';
import { Fabric } from 'office-ui-fabric-react/lib/Fabric';
import {
  IColumn,
  DetailsList,
  DetailsListLayoutMode,
  SelectionMode,
  buildColumns,
  ConstrainMode,
  IDetailsHeaderProps
} from 'office-ui-fabric-react/lib/DetailsList';
import { ScrollablePane, IScrollablePane } from 'office-ui-fabric-react/lib/ScrollablePane';
import { Sticky, StickyPositionType } from 'office-ui-fabric-react/lib/Sticky';
import { TooltipHost, ITooltipHostProps } from 'office-ui-fabric-react/lib/Tooltip';
import { IContextualMenuProps, ContextualMenu, DirectionalHint } from 'office-ui-fabric-react/lib/ContextualMenu';

import { IListProps, IListStyleProps, IListStyles } from './List.types';

import isMobile from 'react-device-detect';

import './List.css';

export interface IListState {
  rows: any[];
  columns: any[];
  isFirstLoad: boolean;
  curPage: number;
  enableSearch: boolean;
  enableSort: boolean;
  searchTxt?: string;
  totalPages?: number;
  isLoading?: boolean;
  sortBy?: string;
  sortDesc?: boolean;
  contextualMenuProps?: IContextualMenuProps;
  overrideDevice: boolean;
}

const getClassNames = classNamesFunction<IListStyleProps, IListStyles>();

export class ListBase extends React.PureComponent<IListProps, IListState> {
  private _scrollablePane: RefObject<IScrollablePane> = createRef<IScrollablePane>();
  constructor(props: IListProps) {
    super(props);
    this.state = {
      rows: [],
      columns: [],
      searchTxt: undefined,
      curPage: 0,
      totalPages: 1,
      isLoading: false,
      isFirstLoad: true,
      enableSearch: this.props.enableSearch != undefined ? this.props.enableSearch : true,
      enableSort: this.props.enableSort != undefined ? this.props.enableSort : true,
      contextualMenuProps: undefined,
      overrideDevice: this.props.overrideDevice != undefined ? this.props.overrideDevice : false
    };
  }

  public render(): JSX.Element {
    const { className, styles, theme } = this.props;
    const classNames = getClassNames(styles, { theme: theme!, className });
    const { columns, rows, enableSearch, contextualMenuProps, overrideDevice } = this.state;

    if (rows) {
      return (
        <Fabric className={`ms-Grid ${classNames.root}`} style={{ height: '100%' }}>
          {enableSearch && (
            <div className="ms-Grid-row" style={{ height: '40px' }}>
              <div className="ms-Grid-col ms-sm12 ms-md12 ms-lg12">
                <SearchBox
                  placeholder="Search"
                  underlined={this.props.underlined}
                  onSearch={newValue => this.onSearch(newValue)}
                  onClear={() => this.onSearch('')}
                  theme={theme}
                  styles={styles}
                />
              </div>
            </div>
          )}
          <div className="ms-Grid-row" style={{ height: '100%' }}>
            <div className="ms-Grid-col ms-sm12 ms-md12 ms-lg12" style={{ position: 'relative', height: '100%' }}>
              <ScrollablePane componentRef={this._scrollablePane} style={{ height: this.props.height }}>
                <DetailsList
                  items={rows}
                  compact={true}
                  columns={columns}
                  componentRef={this.props.componentRef}
                  selectionMode={this.props.selectionMode ? this.props.selectionMode : SelectionMode.none}
                  setKey={this.props.setKey}
                  layoutMode={DetailsListLayoutMode.justified}
                  constrainMode={ConstrainMode.unconstrained}
                  isHeaderVisible={true}
                  selection={this.props.selection}
                  selectionPreservedOnEmptyClick={true}
                  enterModalSelectionOnTouch={true}
                  onRenderMissingItem={this.onDelayedLoadNextPage}
                  onRenderRow={this.props.onRenderRow}
                  onRenderItemColumn={this.props.onRenderItemColumn}
                  onItemContextMenu={this.setRowContextMenu}
                  onRenderDetailsHeader={this.onRenderDetailsHeader}
                  theme={theme}
                  styles={styles}
                />
                {contextualMenuProps != null && (isMobile || overrideDevice) && (
                  <ContextualMenu {...contextualMenuProps} directionalHint={DirectionalHint.bottomLeftEdge} />
                )}
              </ScrollablePane>
            </div>
          </div>
        </Fabric>
      );
    } else {
      return <></>;
    }
  }

  public componentDidMount() {
    this.onDelayedLoadNextPage();
  }

  private onFirstLoad() {
    this.setState({ isFirstLoad: false });

    if (this.props.selection) {
      if (this.props.selectIndexOnInit != undefined) {
        console.log('first load');
        this.props.selection.setIndexSelected(this.props.selectIndexOnInit, true, true);
      }
      if (this.props.selectKeyOnInit != undefined) {
        console.log('first load');
        this.props.selection.setKeySelected(this.props.selectKeyOnInit, true, true);
      }
    }
  }

  private setColumns = (value: any[]): void => {
    this.setState({
      columns: this.props.onCustomBuildColumns ? this.buildCustomColumns(this.props.onCustomBuildColumns) : this.buildColumns(value, true)
    });
  };

  private addColumnDefaults(columns: IColumn[]): IColumn[] {
    columns.forEach(column => {
      column.isCollapsable = false;
      if (this.state.enableSort) {
        column.onColumnClick = this.onColumnClick;
      } else {
        column.isSorted = false;
      }
    });

    return columns;
  }

  private buildCustomColumns(callback: () => IColumn[]) {
    const columns = callback();
    return this.addColumnDefaults(columns);
  }

  private buildColumns(items: any[], canResizeColumns?: boolean) {
    let columns = buildColumns(items, canResizeColumns, undefined, undefined, undefined, undefined);

    columns = this.addColumnDefaults(columns);

    this.props.onBuildColumns ? this.props.onBuildColumns(columns) : null;

    return columns;
  }

  private onSearch(value: string): false {
    this.setState(
      {
        searchTxt: value,
        rows: [],
        curPage: 0,
        totalPages: 1,
        isLoading: false,
        isFirstLoad: true
      },
      () => this.onDelayedLoadNextPage()
    );
    return false;
  }

  private onDelayedLoadNextPage = (): null => {
    let { isLoading } = this.state;

    console.log('Loading next batch of data');
    if (!isLoading) {
      this.setState(
        prevState => ({
          curPage: prevState.curPage + 1,
          isLoading: true
        }),
        () => this.onLoadNextPage()
      );
    }

    return null;
  };

  private onLoadNextPage() {
    let { columns, rows, searchTxt, curPage, sortBy, sortDesc } = this.state;

    const value = searchTxt == null || searchTxt == undefined || searchTxt.trim().length === 0 ? 'null' : searchTxt;

    let newItems: any;

    const url: string = this.props.getUrl(value, curPage, sortBy, sortDesc);

    Axios.get(url).then(response => {
      newItems = this.props.handleResponse(response.data);
      this.setState({ totalPages: newItems.totalPages });
      if (columns.length == 0) {
        this.setColumns(newItems.data);
      }

      if (curPage < newItems.totalPages) {
        newItems.data.push(null);
      }

      this.setState(
        prevState => ({
          rows: prevState.rows ? prevState.rows.slice(0, prevState.rows.length - 1).concat(newItems.data) : newItems.data,
          isLoading: false
        }),
        () => {
          if (this.state.isFirstLoad) {
            this.onFirstLoad();
          }

          console.log('Rows: ', rows.length, ' of ', newItems.recordsTotal);
        }
      );
    });
  }

  private onColumnClick = (ev: React.MouseEvent<HTMLElement>, column: IColumn): void => {
    const { columns } = this.state;

    const newColumns: IColumn[] = columns.slice();
    const currColumn: IColumn = newColumns.filter((currCol: IColumn, idx: number) => {
      return column.key === currCol.key;
    })[0];
    newColumns.forEach((newCol: IColumn) => {
      if (newCol === currColumn) {
        currColumn.isSortedDescending = !currColumn.isSortedDescending;
        currColumn.isSorted = true;
        this.setState({
          sortBy: currColumn.key,
          sortDesc: currColumn.isSortedDescending
        });
      } else {
        newCol.isSorted = false;
        newCol.isSortedDescending = true;
      }
    });

    this.setState(
      {
        rows: [],
        curPage: 0,
        totalPages: 1,
        isLoading: false,
        isFirstLoad: true
      },
      () => this.onDelayedLoadNextPage()
    );
  };

  private setRowContextMenu = (item?: any, index?: number | undefined, ev?: Event | undefined): boolean | void => {
    const { onItemContextMenu } = this.props;

    if (onItemContextMenu != null && index && index > -1) {
      const contextualMenuProps: IContextualMenuProps = onItemContextMenu(item, index, ev);
      contextualMenuProps.onDismiss = this.onDismissRowContextMenu;

      this.setState({
        contextualMenuProps: contextualMenuProps
      });
      return false;
    }
    return true;
  };

  private onDismissRowContextMenu = (): void => {
    this.setState({
      contextualMenuProps: undefined
    });
  };

  private onRenderDetailsHeader = (
    props?: IDetailsHeaderProps,
    defaultRender?: (props?: IDetailsHeaderProps) => JSX.Element | null
  ): JSX.Element | null => {
    return (
      <Sticky stickyPosition={StickyPositionType.Header} isScrollSynced={true}>
        {defaultRender!({
          ...props,
          onRenderColumnHeaderTooltip: this.onRenderColumnHeaderTooltip
        } as IDetailsHeaderProps)}
      </Sticky>
    );
  };

  private onRenderColumnHeaderTooltip = (
    props?: ITooltipHostProps,
    defaultRender?: (props?: ITooltipHostProps) => JSX.Element | null
  ): JSX.Element | null => {
    return <TooltipHost {...props} />;
  };
}
