import * as React from 'react';

import { Fabric } from 'office-ui-fabric-react/lib/Fabric';

import { classNamesFunction } from 'office-ui-fabric-react/lib/Utilities';

import { TextField } from 'office-ui-fabric-react/lib/TextField';

import { CommandBar } from 'office-ui-fabric-react/lib/CommandBar';

import { Toggle } from 'office-ui-fabric-react/lib/Toggle';

import { Panel, PanelType } from 'office-ui-fabric-react/lib/Panel';

import { Icon } from 'office-ui-fabric-react/lib/Icon';

import Axios from 'axios';

import update from 'immutability-helper';

import {
  IMenusAdminFormProps,
  IMenusAdminFormStyleProps,
  IMenusAdminFormStyles,
  IMenuCommand,
  IMenusAdminFormResult
} from './MenusAdminForm.types';

import { MovableOptionsList, IMovableOptionsListElement } from '../MovableOptionsList';

import { Title } from '../Title';

import './MenusAdminForm.css';

const getClassNames = classNamesFunction<IMenusAdminFormStyleProps, IMenusAdminFormStyles>();

export interface IMenusAdminFormState {
  id: string;
  name: string;
  description: string;
  existingCommands: IMenuCommand[];
  movableCommands: IMovableOptionsListElement[];
  previewItems: any[];
  toggleTextMode: boolean;
  showPanel: boolean;
  editableCommand: any;
  error: any;
}

export class MenusAdminFormBase extends React.PureComponent<IMenusAdminFormProps, IMenusAdminFormState> {
  public static defaultProps = {
    labelFormId: 'ID',
    labelFormName: 'Name',
    labelFormDescription: 'Description',
    labelFormCommand: 'Menu Item',
    labelCmdBarBtnSave: 'Save',
    labelCmdBarBtnEdit: 'Edit',
    labelCmdBarBtnCancel: 'Cancel',
    labelFormFieldset: 'Menu Preview',
    labelToggleOn: 'Icon & Text',
    labelToggleOff: 'Icon Only',
    labelFormMenuItemId: 'ID',
    labelFormMenuItemName: 'Name',
    labelFormMenuItemIcon: 'Icon'
  };
  constructor(props: IMenusAdminFormProps) {
    super(props);
    this.state = {
      id: this.props.id ? this.props.id : '',
      name: '',
      description: '',
      existingCommands: [],
      movableCommands: [],
      previewItems: [],
      toggleTextMode: false,
      showPanel: false,
      editableCommand: undefined,
      error: undefined
    };

    this.getFormCommandBarItems = this.getFormCommandBarItems.bind(this);
    this.changePreviewCommandBarMode = this.changePreviewCommandBarMode.bind(this);
    this.onShowPanel = this.onShowPanel.bind(this);
    this.onDismissPanel = this.onDismissPanel.bind(this);
    this.editCommand = this.editCommand.bind(this);
    this.changeEditableCommandIcon = this.changeEditableCommandIcon.bind(this);
  }

  public componentDidMount() {
    this.init();
  }

  public componentDidUpdate(prevProps: IMenusAdminFormProps) {
    if (this.props.id !== prevProps.id) {
      this.init();
    }
  }
  private init() {
    const { id } = this.props;
    if (id) {
      this.componentDidMountEdit();
    } else {
      this.componentDidMountNew();
    }
  }

  public componentDidMountNew() {
    const { commands } = this.props;
    const movableCommandsNew = this.itemsToIMovableOptionsListElements(commands);

    this.setState({ movableCommands: movableCommandsNew, existingCommands: commands });
  }

  public componentDidMountEdit() {
    const { url, id, commands } = this.props;
    const editUrl = `${url}/menus/${id}`;

    console.log('editUrl', editUrl);

    Axios.get(editUrl)
      .then(response => {
        const data = response.data;
        const selectedCommands: IMenuCommand[] = data.items;
        const movableCommandsNew = this.itemsToIMovableOptionsListElements(commands, selectedCommands);
        const previewItemsNew = this.getPreviewCommandBarItems(movableCommandsNew);

        this.setState({
          name: data.name,
          description: data.description,
          existingCommands: commands,
          movableCommands: movableCommandsNew,
          previewItems: previewItemsNew
        });
      })
      .catch(error => this.setState({ error: error }));
  }

  public render(): JSX.Element {
    const {
      className,
      styles,
      theme,
      height,
      labelFormId,
      labelFormName,
      labelFormDescription,
      labelFormCommand,
      title,
      labelFormFieldset,
      labelToggleOn,
      labelToggleOff,
      labelFormMenuItemId,
      labelFormMenuItemName,
      labelFormMenuItemIcon
    } = this.props;
    const { name, description, movableCommands, previewItems, toggleTextMode, showPanel, editableCommand, id } = this.state;
    const classNames = getClassNames(styles, { theme: theme!, className });
    const titleFull = title ? (name ? `${title} [${name}]` : title) : '';

    return (
      <Fabric className={`ms-Grid ${classNames.root}`}>
        <div className="ms-Grid-row" style={{ paddingBottom: '15px' }}>
          <div className="ms-Grid-col ms-sm12 ms-md12">
            <CommandBar items={this.getFormCommandBarItems(this.props.id)} />
          </div>
        </div>
        {title && (
          <div className="ms-Grid-row">
            <div className="ms-Grid-col ms-sm12 ms-md12 ms-lg12">
              <Title title={titleFull} icon={true} />
            </div>
          </div>
        )}
        <div className="ms-Grid-row">
          <div className="ms-Grid-col ms-sm9 ms-md9 ms-lg9">
            <TextField
              key="FormId"
              value={id}
              label={labelFormId}
              required
              onBlur={e => {
                this.setState({ id: e.target.value });
              }}
            />
          </div>
          <div className="ms-Grid-col ms-sm3 ms-md3 ms-lg3" />
        </div>
        <div className="ms-Grid-row">
          <div className="ms-Grid-col ms-sm9 ms-md9 ms-lg9">
            <TextField
              key="FormName"
              value={name}
              label={labelFormName}
              required
              onBlur={e => {
                this.setState({ name: e.target.value });
              }}
            />
          </div>
          <div className="ms-Grid-col ms-sm3 ms-md3 ms-lg3" />
        </div>
        <div className="ms-Grid-row">
          <div className="ms-Grid-col ms-sm9 ms-md9 ms-lg9">
            <TextField
              key="FormDescription"
              value={description}
              label={labelFormDescription}
              required
              onBlur={e => {
                this.setState({ description: e.target.value });
              }}
            />
          </div>
          <div className="ms-Grid-col ms-sm3 ms-md3 ms-lg3" />
        </div>
        <div className="ms-Grid-row" style={{ paddingBottom: '10px' }}>
          <div className="ms-Grid-col ms-sm12 ms-md12 ms-lg12" />
        </div>
        <div className="ms-Grid-row">
          <div className="ms-Grid-col ms-sm9 ms-md9 ms-lg9">
            <fieldset style={{ borderWidth: 'thin', borderColor: '#fff' }}>
              <legend>{labelFormFieldset}</legend>
              <div className="ms-Grid-row">
                <div className="ms-Grid-col ms-sm10 ms-md10 ms-lg10">
                  <CommandBar items={previewItems} />
                </div>
                <div className="ms-Grid-col ms-sm2 ms-md2 ms-lg2" style={{ paddingTop: '5px', textAlign: 'right' }}>
                  <Toggle
                    defaultChecked={toggleTextMode}
                    onText={labelToggleOn}
                    offText={labelToggleOff}
                    onChange={() => this.changePreviewCommandBarMode(toggleTextMode)}
                  />
                </div>
              </div>
            </fieldset>
          </div>
          <div className="ms-Grid-col ms-sm3 ms-md3 ms-lg3" />
        </div>
        <div className="ms-Grid-row" style={{ paddingBottom: '20px' }}>
          <div className="ms-Grid-col ms-sm12 ms-md12 ms-lg12" />
        </div>
        <div className="ms-Grid-row">
          <div className="ms-Grid-col ms-sm9 ms-md9 ms-lg9">
            <MovableOptionsList
              elements={movableCommands}
              property={labelFormCommand}
              height={height ? height : '400px'}
              handleSwapChange={this.handleSwapChange}
              handleMoveUpAndDownChange={this.handleSwapChange}
            />
          </div>
          <div className="ms-Grid-col ms-sm3 ms-md3 ms-lg3" />
        </div>
        {showPanel && (
          <Panel key="ConfigButtonPanel" isOpen={showPanel} type={PanelType.medium} onDismiss={this.onDismissPanel}>
            <div className="ms-Grid">
              <div className="ms-Grid-row">
                <div className="ms-Grid-col ms-sm12 ms-md12 ms-lg12">
                  <Title icon title={`Configure [${editableCommand.name}]`} />
                </div>
              </div>
              <div className="ms-Grid-row">
                <div className="ms-Grid-col ms-sm9 ms-md9 ms-lg9">
                  <div className="ms-Grid">
                    <div className="ms-Grid-row">
                      <div className="ms-Grid-col ms-sm12 ms-md12 ms-lg12">
                        <TextField key="CmdKey" value={editableCommand.key} label={labelFormMenuItemId} disabled />
                      </div>
                    </div>
                    <div className="ms-Grid-row">
                      <div className="ms-Grid-col ms-sm12 ms-md12 ms-lg12">
                        <TextField key="CmdName" value={editableCommand.name} label={labelFormMenuItemName} disabled />
                      </div>
                    </div>
                    <div className="ms-Grid-row">
                      <div className="ms-Grid-col ms-sm12 ms-md12 ms-lg12">
                        <TextField
                          key="CmdIcon"
                          value={editableCommand.iconProps.iconName}
                          label={labelFormMenuItemIcon}
                          required
                          onBlur={e => {
                            this.changeEditableCommandIcon(editableCommand, e.target.value);
                          }}
                        />
                      </div>
                    </div>
                  </div>
                </div>
                <div className="ms-Grid-col ms-sm3 ms-md3 ms-lg3">
                  <div className="menu-icon-preview">
                    <Icon iconName={editableCommand.iconProps.iconName} className="menu-icon-preview-col" />
                  </div>
                </div>
              </div>
            </div>
          </Panel>
        )}
      </Fabric>
    );
  }

  private itemsToIMovableOptionsListElements = (
    allCommands: IMenuCommand[],
    existingCommands: IMenuCommand[] = []
  ): IMovableOptionsListElement[] => {
    const commands: IMovableOptionsListElement[] = [];

    allCommands.forEach(element => {
      const selected = existingCommands.some(({ id }) => id == element.id);
      const mvElement: IMovableOptionsListElement = {
        key: element.id,
        text: element.name,
        selected: selected,
        icon: element.icon
      };
      commands.push(mvElement);
    });

    return commands;
  };

  private getPreviewCommandBarItems = (elements: IMovableOptionsListElement[]): any => {
    const { toggleTextMode } = this.state;
    const previewCmdbarItems: any[] = [];

    elements.forEach(element => {
      if (element.selected) {
        const key = element.key;
        const item = {
          key: key,
          name: element.text,
          iconProps: {
            iconName: element.icon
          },
          iconOnly: !toggleTextMode,
          onClick: () => this.editCommand(key)
        };
        previewCmdbarItems.push(item);
      }
    });

    return previewCmdbarItems;
  };

  private getFormCommandBarItems = (id: any) => {
    const { labelCmdBarBtnSave, labelCmdBarBtnEdit, labelCmdBarBtnCancel } = this.props;

    const formCommandBarItems = [
      {
        key: 'Save',
        name: labelCmdBarBtnSave,
        iconProps: {
          iconName: 'Save'
        },
        onClick: () => this.onSubmit()
      },
      {
        key: 'Edit',
        name: labelCmdBarBtnEdit,
        iconProps: {
          iconName: 'Edit'
        },
        onClick: () => this.onSubmit()
      },
      {
        key: 'Cancel',
        name: labelCmdBarBtnCancel,
        iconProps: {
          iconName: 'Cancel'
        },
        onClick: () => this.props.onCancel()
      }
    ];

    const removeIdx = id ? 0 : 1;
    formCommandBarItems.splice(removeIdx, 1);

    return formCommandBarItems;
  };

  private handleSwapChange = (elements: IMovableOptionsListElement[]): void => {
    const { commands } = this.props;
    const { existingCommands } = this.state;

    const newExistingCommands: IMenuCommand[] = [];

    elements.forEach(element => {
      if (element.selected) {
        const existingCommand: any = this.getElementOnArray(existingCommands, 'id', element.key);
        const icon = existingCommand ? existingCommand.icon : element.icon;
        const command: IMenuCommand = {
          id: element.key,
          name: element.text,
          icon: icon
        };
        newExistingCommands.push(command);
        element.icon = icon;
      } else {
        const originalCommand: any = this.getElementOnArray(commands, 'id', element.key);
        element.icon = originalCommand.icon;
      }
    });

    const previewItemsNew = this.getPreviewCommandBarItems(elements);
    this.setState({ previewItems: previewItemsNew, existingCommands: newExistingCommands });
  };

  private changePreviewCommandBarMode = (mode: boolean): void => {
    const { previewItems } = this.state;
    const previewItemsNew: any[] = [];

    previewItems.forEach(element => {
      const item = {
        key: element.key,
        name: element.name,
        iconProps: {
          iconName: element.iconProps.iconName
        },
        iconOnly: mode,
        onClick: () => this.editCommand(element.key)
      };
      previewItemsNew.push(item);
    });

    this.setState({ toggleTextMode: !mode, previewItems: previewItemsNew });
  };

  private onShowPanel = (): void => {
    this.setState({ showPanel: true });
  };

  private onDismissPanel = (): void => {
    this.setState({ showPanel: false });
  };

  private editCommand = (cmdKey: any): void => {
    const { previewItems } = this.state;
    const command: any = previewItems.find(({ key }) => key === cmdKey);
    this.setState({ editableCommand: command }, () => this.onShowPanel());
  };

  private changeEditableCommandIcon = (editableCommand: any, icon: string): void => {
    const { previewItems, existingCommands } = this.state;
    const key = editableCommand.key;

    const newEditableCommand = {
      key: key,
      name: editableCommand.name,
      iconProps: {
        iconName: icon
      },
      iconOnly: editableCommand.mode,
      onClick: () => this.editCommand(key)
    };

    const previewIndex = previewItems.findIndex(p => p.key === key);
    const command: any = previewItems[previewIndex];
    command.iconProps.iconName = icon;
    const newPreviewItems = update(previewItems, { [previewIndex]: { $set: command } });

    const exCommandIndex = existingCommands.findIndex(p => p.id === key);
    const exCommand: any = existingCommands[exCommandIndex];
    exCommand.icon = icon;
    const newExistingCommands = update(existingCommands, { [exCommandIndex]: { $set: exCommand } });

    this.setState({ editableCommand: newEditableCommand, previewItems: newPreviewItems, existingCommands: newExistingCommands });
  };

  private getElementOnArray = (values: any[], id: string, key: string): any => {
    const index = values.findIndex(p => p[id] === key);
    return values[index];
  };

  private onSubmit = (): void => {
    const { url, onSubmitSuccess, onSubmitError } = this.props;
    const { name, description, toggleTextMode, previewItems, id } = this.state;

    const items: IMenuCommand[] = [];

    previewItems.forEach(element => {
      const item: IMenuCommand = {
        id: element.key,
        name: element.name,
        icon: element.iconProps.iconName
      };
      items.push(item);
    });

    const result: IMenusAdminFormResult = {
      id: id,
      name: name,
      description: description,
      textMode: toggleTextMode,
      items: items
    };

    this.logResult(result);

    const jsonResult = JSON.stringify(result);
    const method = id ? 'post' : 'put';

    console.log('Data Submited to server by', method, jsonResult);

    Axios.request({
      url: `${url}/menus`,
      method: method,
      data: jsonResult,
      headers: {
        Accept: 'application/json',
        'Content-Type': 'application/json'
      }
    })
      .then(response => {
        onSubmitSuccess(response.data);
      })
      .catch(error => onSubmitError(error));
  };

  private logResult = (result: IMenusAdminFormResult): void => {
    let msg = '\n\n------------------------\n';
    msg += '      Form Values       \n';
    msg += '------------------------';
    msg += '\nId:' + result.id;
    msg += '\nName:' + result.name;
    msg += '\nDescription:' + result.description;
    msg += '\n------------------------\n';
    msg += '\n Menu Items';
    msg += '\n------------------------';

    result.items.forEach((row: IMenuCommand, key: number) => {
      msg += '\n   ID: ' + row.id + '\n';
      msg += '\n   Name: ' + row.name + '\n';
      msg += '\n   Icon: ' + row.icon + '\n';
      msg += '------------------------';
    });

    console.log(msg);
  };
}
