import * as React from 'react';

import { Fabric } from 'office-ui-fabric-react/lib/Fabric';

import { ScrollablePane, IScrollablePane } from 'office-ui-fabric-react/lib/ScrollablePane';

import { DetailsList, DetailsListLayoutMode, SelectionMode, Selection, IColumn } from 'office-ui-fabric-react/lib/DetailsList';

import { classNamesFunction, createRef, RefObject } from 'office-ui-fabric-react/lib/Utilities';

import { Label } from 'office-ui-fabric-react/lib/Label';

/*
import { IconButton, IButtonStyles } from 'office-ui-fabric-react/lib/Button';
import { createTheme } from "office-ui-fabric-react/lib/Styling";
*/
import { IconButton } from 'office-ui-fabric-react/lib/Button';

import update from 'immutability-helper';

import { Title } from '../Title';

import './MovableOptionsList.css';

import {
  IMovableOptionsListProps,
  IMovableOptionsListStyleProps,
  IMovableOptionsListStyles,
  IMovableOptionsListElement
} from './MovableOptionsList.types';

const getClassNames = classNamesFunction<IMovableOptionsListStyleProps, IMovableOptionsListStyles>();

/*
const iconsTheme = createTheme({
  semanticColors: {
    disabledBackground: "#ffff00"
  }
});

const iconButtonStyles: IButtonStyles = {
  rootDisabled: {
    backgroundColor: "#ff0000",
  }
};
*/

export const MoveModeEnum = {
  SELECTED_TO_AVAILABLE: 0,
  AVAILABLE_TO_SELECTED: 1
};

export const MoveDirectionEnum = {
  UP: -1,
  DOWN: 1
};

export interface IMovableOptionsListState {
  isAvailableSelected: boolean;
  isSelectedSelected: boolean;
  availableElems: IMovableOptionsListElement[];
  selectedElems: IMovableOptionsListElement[];
  error: any;
}

export class MovableOptionsListBase extends React.PureComponent<IMovableOptionsListProps, IMovableOptionsListState> {
  private scrollablePaneAvailable: RefObject<IScrollablePane> = createRef<IScrollablePane>();
  private scrollablePaneSelected: RefObject<IScrollablePane> = createRef<IScrollablePane>();
  private selectionAvailable: Selection;
  private selectionSelected: Selection;

  private columns: IColumn[] = [
    {
      key: 'Property',
      name: 'Property',
      fieldName: 'text',
      minWidth: 70,
      maxWidth: 90
    }
  ];

  constructor(props: IMovableOptionsListProps) {
    super(props);
    this.state = {
      availableElems: [],
      selectedElems: [],
      error: undefined,
      isAvailableSelected: false,
      isSelectedSelected: false
    };

    this.swapElements = this.swapElements.bind(this);
    this.moveSelected = this.moveSelected.bind(this);
    this.checkElements = this.checkElements.bind(this);
    this.handleSwapChange = this.handleSwapChange.bind(this);
    this.handleMoveUpAndDownChange = this.handleMoveUpAndDownChange.bind(this);

    this.selectionAvailable = new Selection({
      onSelectionChanged: () => {
        this.setState({ isAvailableSelected: this.selectionAvailable.getSelection().length > 0 });
      }
    });

    this.selectionSelected = new Selection({
      onSelectionChanged: () => {
        this.setState({ isSelectedSelected: this.selectionSelected.getSelection().length > 0 });
      }
    });
  }

  public componentDidMount() {
    const { property } = this.props;
    if (property) {
      this.columns[0].name = property;
    }
    this.checkElements();
  }

  public checkElements() {
    const { elements } = this.props;
    const availableElems: IMovableOptionsListElement[] = [];
    const selectedElems: IMovableOptionsListElement[] = [];

    if (elements) {
      for (let elem of elements) {
        const elementsPointer = elem.selected ? selectedElems : availableElems;
        elementsPointer.push(elem);
      }
    }
    this.setState({ availableElems: availableElems, selectedElems: selectedElems });
  }

  public componentDidUpdate(prevProps: IMovableOptionsListProps) {
    if (this.props.elements !== prevProps.elements) {
      this.checkElements();
    }
  }

  public render(): JSX.Element {
    const { className, styles, theme, availableLabel, selectedLabel, title, disableUpAndDown } = this.props;
    const classNames = getClassNames(styles, { theme: theme!, className });
    const { availableElems, selectedElems } = this.state;

    return (
      <Fabric className={`ms-Grid ${classNames.root}`}>
        {title && <Title title={title} icon={true} />}

        <div className="ms-Grid-row">
          <div className="ms-Grid-col ms-sm5 ms-md5 ms-lg5">
            <Label className="ms-font-l ms-fontWeight-regular">{availableLabel ? availableLabel : 'Available'}</Label>
          </div>
          <div className="ms-Grid-col ms-sm1 ms-md1 ms-lg1" />
          <div className="ms-Grid-col ms-sm5 ms-md5 ms-lg5">
            <Label className="ms-font-l  ms-fontWeight-regular">{selectedLabel ? selectedLabel : 'Selected'}</Label>
          </div>
          <div className="ms-Grid-col ms-sm1 ms-md1 ms-lg1" />
        </div>

        <div className="ms-Grid-row ml-nl-md">
          <div className="ms-Grid-col ms-sm12 ms-md12 ms-lg12" />
        </div>

        <div className="ms-Grid-row">
          <div className="ms-Grid-col ms-sm5 ms-md5 ms-lg5" style={{ position: 'relative' }}>
            <ScrollablePane componentRef={this.scrollablePaneAvailable} style={{ height: this.props.height }}>
              <DetailsList
                key="availableListKey"
                columns={this.columns}
                items={availableElems}
                selectionMode={SelectionMode.multiple}
                setKey="setAvailable"
                layoutMode={DetailsListLayoutMode.justified}
                isHeaderVisible={true}
                selectionPreservedOnEmptyClick={true}
                enterModalSelectionOnTouch={true}
                selection={this.selectionAvailable}
              />
            </ScrollablePane>
          </div>
          <div className="ms-Grid-col ms-sm1 ms-md1 ms-lg1">
            <div className="ms-Grid">
              <div className="ms-Grid-row">
                <div className="ms-Grid-col ms-sm12 ms-md12 ms-lg12" style={{ textAlign: 'center' }}>
                  <IconButton
                    iconProps={{ iconName: 'ChevronLeft' }}
                    disabled={!this.state.isSelectedSelected}
                    onClick={() => this.swapElements(MoveModeEnum.SELECTED_TO_AVAILABLE)}
                    //styles={iconButtonStyles}
                  />
                </div>
              </div>
              <div className="ms-Grid-row">
                <div className="ms-Grid-col ms-sm12 ms-md12 ms-lg12" style={{ textAlign: 'center' }}>
                  <IconButton
                    iconProps={{ iconName: 'ChevronRight' }}
                    disabled={!this.state.isAvailableSelected}
                    onClick={() => this.swapElements(MoveModeEnum.AVAILABLE_TO_SELECTED)}
                    //theme={iconsTheme}
                  />
                </div>
              </div>
            </div>
          </div>
          <div className="ms-Grid-col ms-sm5 ms-md5 ms-lg5" style={{ position: 'relative' }}>
            <ScrollablePane componentRef={this.scrollablePaneSelected} style={{ height: this.props.height }}>
              <DetailsList
                key="selectedListKey"
                columns={this.columns}
                items={selectedElems}
                selectionMode={SelectionMode.multiple}
                setKey="setSelected"
                layoutMode={DetailsListLayoutMode.justified}
                isHeaderVisible={true}
                selectionPreservedOnEmptyClick={true}
                enterModalSelectionOnTouch={true}
                selection={this.selectionSelected}
              />
            </ScrollablePane>
          </div>
          <div className="ms-Grid-col ms-sm1 ms-md1 ms-lg1">
            {!disableUpAndDown && (
              <div className="ms-Grid">
                <div className="ms-Grid-row">
                  <div className="ms-Grid-col ms-sm12 ms-md12 ms-lg12">
                    <IconButton
                      iconProps={{ iconName: 'ChevronUp' }}
                      disabled={!this.state.isSelectedSelected}
                      onClick={() => this.moveSelected(MoveDirectionEnum.UP)}
                    />
                  </div>
                </div>
                <div className="ms-Grid-row">
                  <div className="ms-Grid-col ms-sm12 ms-md12 ms-lg12">
                    <IconButton
                      iconProps={{ iconName: 'ChevronDown' }}
                      disabled={!this.state.isSelectedSelected}
                      onClick={() => this.moveSelected(MoveDirectionEnum.DOWN)}
                    />
                  </div>
                </div>
              </div>
            )}
          </div>
        </div>
      </Fabric>
    );
  }

  private swapElements = (mode: number) => {
    const selection = mode === MoveModeEnum.SELECTED_TO_AVAILABLE ? this.selectionSelected : this.selectionAvailable;
    const selectedItems: any[] = selection.getSelection();
    const { availableElems, selectedElems } = this.state;

    let selectedElemsCopy: any[] = [];
    let availableElemsCopy: any[] = [];

    if (selectedItems.length > 0) {
      for (let item of selectedItems) {
        const selectedListOrigin = selectedElemsCopy.length > 0 ? selectedElemsCopy : selectedElems;
        const availableListOrigin = availableElemsCopy.length > 0 ? availableElemsCopy : availableElems;

        item.selected = mode === MoveModeEnum.SELECTED_TO_AVAILABLE ? false : true;

        const listToSearchIndex = mode === MoveModeEnum.SELECTED_TO_AVAILABLE ? selectedListOrigin : availableListOrigin;
        const indexToRemove = listToSearchIndex.findIndex(value => value.key === item.key);

        if (indexToRemove >= 0) {
          if (mode === MoveModeEnum.SELECTED_TO_AVAILABLE) {
            selectedElemsCopy = update(selectedListOrigin, { $splice: [[indexToRemove, 1]] });
            availableElemsCopy = update(availableListOrigin, { $push: [item] });
          } else {
            availableElemsCopy = update(availableListOrigin, { $splice: [[indexToRemove, 1]] });
            selectedElemsCopy = update(selectedListOrigin, { $push: [item] });
          }
        }
      }
      this.setState({ selectedElems: selectedElemsCopy, availableElems: availableElemsCopy }, () => this.handleSwapChange());
    }
  };

  private moveSelected = (pos: number): void => {
    const { selectedElems } = this.state;

    const selectedItems: any[] = this.selectionSelected.getSelection();

    if (selectedItems.length === 1) {
      const item = selectedItems[0];
      const index = selectedElems.findIndex(value => value.key === item.key);

      if (index >= 0) {
        const element = selectedElems[index];
        let selectedElemsCopy = update(selectedElems, { $splice: [[index, 1]] });
        selectedElemsCopy = update(selectedElemsCopy, { $splice: [[index + pos, 0, element]] });
        this.setState({ selectedElems: selectedElemsCopy }, () => this.handleMoveUpAndDownChange());
      }
    }
  };

  private handleSwapChange = (): void => {
    const { selectedElems } = this.state;
    const { handleSwapChange } = this.props;

    if (handleSwapChange) {
      handleSwapChange(selectedElems);
    }
  };

  private handleMoveUpAndDownChange = (): void => {
    const { selectedElems } = this.state;
    const { handleMoveUpAndDownChange } = this.props;

    if (handleMoveUpAndDownChange) {
      handleMoveUpAndDownChange(selectedElems);
    }
  };
}
