import * as React from 'react';

import Axios from 'axios';

import { Fabric } from 'office-ui-fabric-react/lib/Fabric';
import { classNamesFunction } from 'office-ui-fabric-react/lib/Utilities';
import { CommandBar } from 'office-ui-fabric-react/lib/CommandBar';
import { TextField } from 'office-ui-fabric-react/lib/TextField';
import { Dropdown, IDropdownOption } from 'office-ui-fabric-react/lib/Dropdown';
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome';

import {
  IQuickButtonFormProps,
  IQuickButtonFormStyleProps,
  IQuickButtonFormStyles /* IQuickButtonFormResult */,
  getFAIcon
} from './QuickButtonForm.types';

import { Title } from '../Title';

import './QuickButtonForm.css';

const getClassNames = classNamesFunction<IQuickButtonFormStyleProps, IQuickButtonFormStyles>();

export interface IQuickButtonFormState {
  id?: number;
  name?: string;
  selectedTemplateKey?: number;
  icon?: string;
  templates: any[];
  error: any;
}

export class QuickButtonFormBase extends React.PureComponent<IQuickButtonFormProps, IQuickButtonFormState> {
  //private result: IQuickButtonFormResult = { name: '', template: '', icon: '' };
  public static defaultProps = {
    labelCmdBarBtnSave: 'Save',
    labelCmdBarBtnEdit: 'Edit',
    labelCmdBarBtnCancel: 'Cancel',
    labelFormTitleName: 'Button Label',
    labelFormTitleTemplate: 'Search Template',
    labelFormTitleIcon: 'Name of Icon'
  };

  constructor(props: IQuickButtonFormProps) {
    super(props);
    this.state = {
      id: this.props.id,
      name: '',
      templates: [],
      icon: '',
      selectedTemplateKey: undefined,
      error: undefined
    };
    this.onSubmit = this.onSubmit.bind(this);
  }

  public componentDidMount() {
    this.init();
  }

  public componentDidUpdate(prevProps: IQuickButtonFormProps) {
    if (this.props.id !== prevProps.id) {
      this.init();
    }
  }

  public componentDidMountNew() {
    const { url } = this.props;
    const templatesUrl = `${url}/templates`;

    this.loadTemplates(templatesUrl);
  }

  private init() {
    const { id } = this.props;
    if (id) {
      this.componentDidMountEdit();
    } else {
      this.componentDidMountNew();
    }
  }

  private loadTemplates = (url: string, selectedId?: any) => {
    Axios.get(url)
      .then(response => {
        const values: any[] = [];
        response.data.data.forEach((element: any) => {
          const isSelected = selectedId ? element.id === selectedId : false;
          const value = { key: element.id, text: element.name, selected: isSelected };
          values.push(value);
        });
        this.setState({
          selectedTemplateKey: selectedId,
          templates: values
        });
      })
      .catch(error => this.setState({ error: error }));
  };

  public componentDidMountEdit() {
    const { url, id } = this.props;
    const editUrl = `${url}/quickbuttons/${id}`;
    const templatesUrl = `${url}/templates`;

    console.log(editUrl);

    Axios.get(editUrl)
      .then(response => {
        /*         this.result.name = response.data.name;
        this.result.id = response.data.id;
        this.result.icon = response.data.icon;
        this.result.template = response.data.template; */

        this.setState(
          {
            name: response.data.name,
            id: response.data.id,
            icon: response.data.icon
          },
          () => this.loadTemplates(templatesUrl, response.data.template)
        );
      })
      .catch(error => this.setState({ error: error }));
  }

  public render(): JSX.Element {
    const { className, styles, theme, id, title, labelFormTitleName, labelFormTitleIcon, labelFormTitleTemplate } = this.props;
    const classNames = getClassNames(styles, { theme: theme!, className });
    const { name, templates, icon, selectedTemplateKey } = this.state;
    const titleFull = title ? (name ? `${title} [${name}]` : title) : '';

    return (
      <Fabric className={`ms-Grid ${classNames.root}`}>
        <div className="ms-Grid-row">
          <div className="ms-Grid-col ms-sm12 ms-md12 ms-lg12">
            <CommandBar items={this.getFormCommandBarItems(id)} />
          </div>
        </div>
        <div className="ms-Grid-row qb-nl-lg">
          <div className="ms-Grid-col ms-sm12 ms-md12 ms-lg12" />
        </div>

        {title && (
          <div className="ms-Grid-row">
            <div className="ms-Grid-col ms-sm12 ms-md12 ms-lg12">
              <Title title={titleFull} icon={true} />
            </div>
          </div>
        )}
        <div className="ms-Grid-row">
          <div className="ms-Grid-col ms-sm10 ms-md10 ms-lg10">
            <div className="ms-Grid">
              <div className="ms-Grid-row">
                <div className="ms-Grid-col ms-sm12 ms-md12 ms-lg12">
                  <TextField
                    key="TemplateName"
                    value={name}
                    label={labelFormTitleName}
                    required
                    onChange={(e, value) => {
                      /*  this.result.name = e.target.value; */
                      this.setState({ name: value });
                    }}
                  />
                </div>
              </div>
              <div className="ms-Grid-row">
                <div className="ms-Grid-col ms-sm12 ms-md12 ms-lg12">
                  <Dropdown
                    key="TemplatesDropdown"
                    options={templates}
                    label={labelFormTitleTemplate}
                    required={true}
                    onChange={(event, option?: IDropdownOption) => {
                      //this.result.template = option.key;
                      this.setState({ selectedTemplateKey: option!.key as number });
                    }}
                    defaultSelectedKey={selectedTemplateKey}
                  />
                </div>
              </div>
              <div className="ms-Grid-row">
                <div className="ms-Grid-col ms-sm12 ms-md12 ms-lg12">
                  <TextField
                    key="IconName"
                    value={icon}
                    label={labelFormTitleIcon}
                    required
                    onChange={(e, icon) => {
                      // this.result.icon = icon || '';
                      this.setState({ icon: icon });
                    }}
                  />
                </div>
              </div>
            </div>
          </div>
          <div className="ms-Grid-col ms-sm2 ms-md2 ms-lg2 icon-preview-col">
            <FontAwesomeIcon className="icon-preview" icon={getFAIcon(icon)} />
          </div>
        </div>
      </Fabric>
    );
  }

  private getFormCommandBarItems = (id: any) => {
    const { labelCmdBarBtnSave, labelCmdBarBtnEdit, labelCmdBarBtnCancel } = this.props;

    const formCommandBarItems = [
      {
        key: 'Save',
        name: labelCmdBarBtnSave,
        iconProps: {
          iconName: 'Save'
        },
        onClick: () => this.onSubmit()
      },
      {
        key: 'Edit',
        name: labelCmdBarBtnEdit,
        iconProps: {
          iconName: 'Edit'
        },
        onClick: () => this.onSubmit()
      },
      {
        key: 'Cancel',
        name: labelCmdBarBtnCancel,
        iconProps: {
          iconName: 'Cancel'
        },
        onClick: () => this.props.onCancel()
      }
    ];

    const removeIdx = id ? 0 : 1;
    formCommandBarItems.splice(removeIdx, 1);

    return formCommandBarItems;
  };

  private logResult = () => {
    let msg = '\n----------------------';
    msg += '\n Quick Button form Details';
    msg += '\n----------------------';
    msg += '\n';
    if (this.state.id) {
      msg += `\n\tId: ${this.state.id}`;
    }

    msg += `\n\tName: ${this.state.name}`;
    msg += `\n\tTemplate ID: ${this.state.selectedTemplateKey}`;
    msg += `\n\tIcon: ${this.state.icon}`;
    msg += '\n----------------------';

    console.log(msg);
  };

  private onSubmit() {
    const { id, url, onSubmitSuccess, onSubmitError } = this.props;

    if (!this.state.name || !this.state.selectedTemplateKey || !this.state.icon) {
      console.log('result empty');
      return;
    }

    this.logResult();

    const jsonResult = JSON.stringify({
      id: this.state.id,
      name: this.state.name,
      template: this.state.selectedTemplateKey,
      icon: this.state.icon
    });

    const method = id ? 'post' : 'put';
    const apiUrl = id ? `${url}/quickbuttons/${id}` : `${url}/quickbuttons`;

    console.log('Data Submited to server by', method, jsonResult);

    Axios.request({
      url: apiUrl,
      method: method,
      data: jsonResult,
      headers: {
        Accept: 'application/json',
        'Content-Type': 'application/json'
      }
    })
      .then(response => {
        onSubmitSuccess(response.data);
      })
      .catch(error => onSubmitError(error));
  }
}
