import * as React from 'react';

import { createRef, classNamesFunction } from 'office-ui-fabric-react/lib/Utilities';

import { Callout } from 'office-ui-fabric-react/lib/Callout';
import { SearchBox as FabricSearchBox } from 'office-ui-fabric-react/lib/SearchBox';
import { GroupedList, IGroup } from 'office-ui-fabric-react/lib/components/GroupedList/index';
import { DetailsRow } from 'office-ui-fabric-react/lib/components/DetailsList/DetailsRow';
import { Selection, SelectionMode } from 'office-ui-fabric-react/lib/utilities/selection/index';

import { ISearchBoxProps, ISearchBoxStyleProps, ISearchBoxStyles, ISearchBoxGroupItem } from './SearchBox.types';

import './SearchBox.css';
import { IColumn } from 'office-ui-fabric-react/lib/DetailsList';

export interface ISearchBoxState {
  isSearchCalloutVisible: boolean;
  items: any[];
  groups: IGroup[];
  input?: string;
}

const getClassNames = classNamesFunction<ISearchBoxStyleProps, ISearchBoxStyles>();

export class SearchBoxBase extends React.PureComponent<ISearchBoxProps, ISearchBoxState> {
  private _searchElement = createRef<HTMLElement>();
  private _selection: Selection;

  constructor(props: any) {
    super(props);
    this.state = {
      isSearchCalloutVisible: false,
      items: [],
      groups: []
    };

    this._onRenderCell = this._onRenderCell.bind(this);

    this._selection = new Selection({
      onSelectionChanged: () => {
        this.props.onSelectionChanged ? this.props.onSelectionChanged(this._selection.getSelection()) : null;
      }
    });
  }

  static getDerivedStateFromProps(props: ISearchBoxProps, state: ISearchBoxState) {
    let groups: any[] = [];
    let items: any[] = [];
    let itemIdx = 0;

    if (props.groupItems)
      props.groupItems!.forEach(element => {
        element.items.forEach(item => {
          item.onItemClick = element.onItemClick;
        });

        groups = [...groups, { key: element.key, name: element.name, startIndex: itemIdx, count: element.items.length }];
        items = [...items, ...element.items];
        itemIdx = itemIdx + element.items.length;
      });

    return { groups: groups, items: items };
  }

  private onFocus = (): void => {
    this.setState({
      isSearchCalloutVisible: true
    });

    this.props.onFocus ? this.props.onFocus() : null;
  };

  private onBlur = (): void => {
    this.props.onBlur ? this.props.onBlur() : null;
  };

  private onSearch = (search: string): void => {
    this._onSearchCalloutDismiss();

    this.props.onSearch(search);
  };

  private _onSearchCalloutDismiss = (): void => {
    this.setState({
      isSearchCalloutVisible: false
    });
  };

  private onItemClick(item: any): void {
    this.setState({ input: item.input, isSearchCalloutVisible: false });
    item.onItemClick(item);
  }

  private onRenderRow(item: ISearchBoxGroupItem): IColumn[] {
    return item.onRenderColumns(item);
  }

  private _onRenderCell(nestingDepth?: number, item?: any, itemIndex?: number): JSX.Element {
    const { _selection: selection } = this;
    return (
      <div onClick={event => this.onItemClick(item)}>
        <DetailsRow
          columns={this.onRenderRow(item)}
          groupNestingDepth={nestingDepth}
          item={item}
          itemIndex={itemIndex!}
          selection={selection}
          selectionMode={SelectionMode.none}
          className="sbRows"
          indentWidth={this.props.indentWidth}
        />
      </div>
    );
  }

  public render(): JSX.Element {
    const { className, styles, theme } = this.props;
    const classNames = getClassNames(styles, { theme: theme!, className });

    return (
      <div className={`${classNames.root}`}>
        <div ref={this._searchElement}>
          <FabricSearchBox
            placeholder={this.props.placeholder ? this.props.placeholder : 'Search'}
            value={this.state.input ? this.state.input : this.props.value}
            underlined={this.props.underlined}
            onFocus={this.onFocus}
            onBlur={this.onBlur}
            onChange={this.props.onChange}
            onClear={this.props.onClear}
            onSearch={this.onSearch}
            theme={theme}
            styles={styles}
          />
        </div>

        <Callout
          ariaLabelledBy={'callout-label-1'}
          ariaDescribedBy={'callout-description-1'}
          role={'alertdialog'}
          gapSpace={1}
          isBeakVisible={false}
          onDismiss={this._onSearchCalloutDismiss}
          setInitialFocus={true}
          hidden={!this.state.isSearchCalloutVisible}
          target={this._searchElement.value}
          theme={theme}
          styles={styles}
        >
          <GroupedList
            items={this.state.items || []}
            onRenderCell={this._onRenderCell}
            selection={this._selection}
            selectionMode={SelectionMode.none}
            groups={this.state.groups}
            groupProps={{
              showEmptyGroups: true
            }}
            theme={theme}
            styles={styles}
          />
        </Callout>
      </div>
    );
  }
}

export default SearchBoxBase;
