import * as React from 'react';

import Axios from 'axios';

import { Fabric } from 'office-ui-fabric-react/lib/Fabric';
import { classNamesFunction } from 'office-ui-fabric-react/lib/Utilities';
import { Label } from 'office-ui-fabric-react/lib/Label';
import { CommandBar } from 'office-ui-fabric-react/lib/CommandBar';
import { TextField } from 'office-ui-fabric-react/lib/TextField';
import { Dropdown, IDropdownOption } from 'office-ui-fabric-react/lib/Dropdown';
import { DatePicker } from 'office-ui-fabric-react/lib/DatePicker';
import { Panel, PanelType } from 'office-ui-fabric-react/lib/Panel';
import { ChoiceGroup, IChoiceGroupOption } from 'office-ui-fabric-react/lib/ChoiceGroup';
import { Checkbox } from 'office-ui-fabric-react/lib/Checkbox';
import { SelectionMode, Selection } from 'office-ui-fabric-react/lib/DetailsList';
import { ActionButton, DefaultButton, PrimaryButton } from 'office-ui-fabric-react/lib/Button';
import { Dialog, DialogType, DialogFooter } from 'office-ui-fabric-react/lib/Dialog';
import update from 'immutability-helper';
import { List } from '../List';
import { MovableOptionsList, IMovableOptionsListElement } from '../MovableOptionsList';
import { Title } from '../Title';
import { FieldTypeEnum } from '../../Utils/Utils.types';
import { cloneMap, deepClone } from '../../Utils/Utils';
import { MessageBar, MessageBarType } from 'office-ui-fabric-react/lib/MessageBar';

import './TemplateAdminForm.css';

import {
  ITemplateAdminFormProps,
  ITemplateAdminFormStyleProps,
  ITemplateAdminFormStyles,
  OperatorsSettingsTypeEnum,
  ITemplateAdminMainRow,
  ITemplateAdminPropertiesRow,
  ITemplateAdminFormObject,
  IFieldValue,
  FormModeEnum
} from './TemplateAdminForm.types';
import { Stack } from 'office-ui-fabric-react';

const getClassNames = classNamesFunction<ITemplateAdminFormStyleProps, ITemplateAdminFormStyles>();

export interface IInitialState {
  name: string;
  main: any[];
  properties: any[];
  displayProperties: IMovableOptionsListElement[];
  movableMainValues: Map<number, IMovableOptionsListElement[]>;
  fieldProperties: IDropdownOption[];
  operatorsProperties: Map<number, IDropdownOption[]>;
  hideConfirmMainChange: boolean;
}

export interface ITemplateAdminFormState {
  id: string;
  name: string;
  main: any[];
  properties: any[];
  propertiesTemp: any[];
  showSettingsPanel: boolean[];
  showMovablePanel: boolean[];
  displayPropertiesPanel: boolean;
  menusPanel: boolean;
  addPropertiesPanel: boolean;
  settingsOperators: Map<number, boolean[]>;
  settingsOperatorsPropertyType: Map<number, any>;
  settingsOperatorsSelectionType: string[];
  displayProperties: IMovableOptionsListElement[];
  movableMainValues: Map<number, IMovableOptionsListElement[]>;
  error: any;
  hideDeleteDialogs: boolean[];
  hideResetDialog: boolean;
  previewItems: any[];
  previewLabel: string | undefined;
  fieldProperties: IDropdownOption[];
  operatorsProperties: Map<number, IDropdownOption[]>;
  hideConfirmMainChange: boolean;
  validateMainFieldsError: string | undefined;
  validatePropertiesFieldsError: boolean;
  validateDisplayPropertiesError: boolean;
}

export interface IMainLastState {
  main: any;
  rows: Map<number, ITemplateAdminMainRow>;
  movableMainValues: Map<number, IMovableOptionsListElement[]>;
}

export class TemplateAdminFormBase extends React.PureComponent<ITemplateAdminFormProps, ITemplateAdminFormState> {
  public static defaultProps = {
    useName: false,
    allowDisplayProperties: true,
    allowMenus: true,
    labelTitleNew: 'Create New Template',
    labelTitleEdit: 'Edit Template',
    labelFormTemplateName: 'Template Name',
    labelDialogTitleDelete: 'Delete',
    labelDialogSubtextPrefixDelete: 'This action will Delete the Property',
    labelDialogSubtextSuffixDelete: 'from the Template. Are you sure you want to proceed?',
    labelDialogBtnDelete: 'Delete',
    labelDialogBtnCancel: 'Cancel',
    labelPanelSettingsPropertyTitle: 'Property Is',
    labelPanelSettingsOperatorsTitle: 'Operators',
    labelPanelSettingsBtnUseAll: 'Use All',
    labelPanelSettingsBtnUseSelected: 'Use Selected',
    labelPanelSettingsEquals: 'Equals',
    labelPanelSettingsStartsWith: 'Starts With',
    labelPanelSettingsContains: 'Contains',
    labelPanelSettingsIsEmpty: 'Is Empty',
    labelPanelSettingsGreaterThan: 'Greater Than',
    labelPanelSettingsLessThan: 'Less Than',
    labelPanelSettingsWithin: 'Within',
    labelPanelSettingsIncludes: 'Includes',
    labelDialogTitleReset: 'Reset',
    labelDialogSubtextReset: 'This action will reset the Form to its Initial values. Are you sure you want to proceed?',
    labelDialogBtnReset: 'Reset',
    labelPanelDisplayPropertiesTitle: 'Display Properties',
    labelPanelAddPropertiesTitle: 'Add Properties',
    labelPanelMenusTitle: 'Add Menu',
    labelPanelAddBtnAdd: 'Add',
    labelPanelAddBtnCancel: 'Cancel',
    labelIconOperatorDelete: ' Delete',
    labelIconOperatorSetting: ' Settings',
    labelIconOperatorUp: ' Up',
    labelIconOperatorDown: ' Down',
    labelCmdBarBtnSave: 'Save',
    labelCmdBarBtnEdit: 'Edit',
    labelCmdBarBtnCancel: 'Cancel',
    labelCmdBarFarBtnAdd: 'Add Property',
    labelCmdBarFarBtnReset: 'Reset',
    labelCmdBarFarBtnResults: 'Display results',
    labelMovListAvailable: 'Available',
    labelMovListSelected: 'Selected',
    labelFormFieldset: 'Selected Menu Preview',
    labelMenuRemoveSelection: 'Remove Menu',
    labelDialogTitleConfirmMainChange: 'Please confirm change!',
    labelDialogSubtextConfirmMainChange: 'This change will reset the Properties Data. Are you sure you want to proceed?',
    labelDialogBtnConfirmMainChange: 'Confirm',
    labelMainValidationMessage: 'The following fields are required: ',
    labelPropertiesValidationMessage: 'There are no Properties defined.',
    labelDisplayPropertiesValidationMessage: 'No Display Property has been selected.'
  };
  private mode: FormModeEnum;
  private initialState: IInitialState = {
    name: '',
    main: [],
    properties: [],
    displayProperties: [],
    fieldProperties: [],
    operatorsProperties: new Map<number, IDropdownOption[]>(),
    movableMainValues: new Map<number, IMovableOptionsListElement[]>(),
    hideConfirmMainChange: true
  };
  private result: ITemplateAdminFormObject = {
    id: '',
    name: '',
    main: new Map<number, ITemplateAdminMainRow>(),
    properties: new Map<number, ITemplateAdminPropertiesRow>(),
    displayProperties: [],
    menuId: ''
  };
  private emptyState: ITemplateAdminFormState;

  private selection: Selection;

  private menusSelection: Selection;

  private url: string;

  private canLoadDisplayProperties: boolean = true;

  private canResetMovableResult: boolean = true;

  private delayedTimeout = 10000;

  private lastMainState: IMainLastState = {
    main: undefined,
    rows: new Map<number, ITemplateAdminMainRow>(),
    movableMainValues: new Map<number, IMovableOptionsListElement[]>()
  };

  constructor(props: ITemplateAdminFormProps) {
    super(props);

    const { id } = this.props;
    const idValue = id ? id : '';

    this.mode = id ? FormModeEnum.EDIT : FormModeEnum.NEW;
    this.url = `${this.props.baseUrl}/templates`;

    this.emptyState = {
      id: idValue,
      name: '',
      main: [],
      properties: [],
      propertiesTemp: [],
      showSettingsPanel: [],
      showMovablePanel: [],
      displayPropertiesPanel: false,
      addPropertiesPanel: false,
      menusPanel: false,
      settingsOperators: new Map<number, boolean[]>(),
      settingsOperatorsPropertyType: new Map<number, number>(),
      settingsOperatorsSelectionType: [],
      displayProperties: [],
      movableMainValues: new Map<number, IMovableOptionsListElement[]>(),
      error: undefined,
      hideDeleteDialogs: [],
      hideResetDialog: true,
      previewItems: [],
      previewLabel: this.props.labelFormFieldset,
      fieldProperties: [],
      operatorsProperties: new Map<number, IDropdownOption[]>(),
      hideConfirmMainChange: true,
      validateMainFieldsError: undefined,
      validatePropertiesFieldsError: false,
      validateDisplayPropertiesError: false
    };

    this.state = this.emptyState;

    this.removeProperty = this.removeProperty.bind(this);
    this.onShowSettingsPanel = this.onShowSettingsPanel.bind(this);
    this.onChangeSettingsOperators = this.onChangeSettingsOperators.bind(this);
    this.isSettingsOperatorChecked = this.isSettingsOperatorChecked.bind(this);
    this.onChangeSettingsOperatorType = this.onChangeSettingsOperatorType.bind(this);
    this.onChangePropertyType = this.onChangePropertyType.bind(this);
    this.getFormCommandBarItems = this.getFormCommandBarItems.bind(this);
    this.onShowAddPropertiesPanel = this.onShowAddPropertiesPanel.bind(this);
    this.onShowMenusPanel = this.onShowMenusPanel.bind(this);
    this.onDisplayPropertiesSelectionChanged = this.onDisplayPropertiesSelectionChanged.bind(this);
    this.onAddProperties = this.onAddProperties.bind(this);
    this.reset = this.reset.bind(this);
    this.componentDidMountInEditMode = this.componentDidMountInEditMode.bind(this);
    this.componentDidMountInNewMode = this.componentDidMountInNewMode.bind(this);
    this.getPropertiesUrl = this.getPropertiesUrl.bind(this);
    this.getMenusUrl = this.getMenusUrl.bind(this);
    this.onSubmit = this.onSubmit.bind(this);
    this.showResetDialog = this.showResetDialog.bind(this);
    this.closeResetDialog = this.closeResetDialog.bind(this);
    this.onDismissMovablePanel = this.onDismissMovablePanel.bind(this);
    this.onShowMovablePanel = this.onShowMovablePanel.bind(this);
    this.updateChildMainField = this.updateChildMainField.bind(this);
    this.populatePreviewItems = this.populatePreviewItems.bind(this);
    this.setResultPropertiesValues = this.setResultPropertiesValues.bind(this);
    this.getFieldTypeForProperty = this.getFieldTypeForProperty.bind(this);
    this.getOperationsForProperty = this.getOperationsForProperty.bind(this);
    this.getDynamicTranslationInternal = this.getDynamicTranslationInternal.bind(this);
    this.getElementOperatorProperties = this.getElementOperatorProperties.bind(this);
    this.resetValuesDueToMainchange = this.resetValuesDueToMainchange.bind(this);
    this.openConfirmMainChange = this.openConfirmMainChange.bind(this);
    this.restoreLastMainState = this.restoreLastMainState.bind(this);

    this.selection = new Selection({
      onSelectionChanged: () => {
        this.onDisplayPropertiesSelectionChanged(this.selection.getSelection());
      }
    });

    this.menusSelection = new Selection({
      onSelectionChanged: () => {
        this.onMenusSelectionChanged(this.menusSelection.getSelection());
      }
    });
  }

  public componentDidMount() {
    this.init();
  }

  public componentDidUpdate(prevProps: ITemplateAdminFormProps) {
    if (this.props.id !== prevProps.id) {
      this.init();
    }
  }

  private init() {
    if (this.mode === FormModeEnum.EDIT) {
      this.componentDidMountInEditMode();
    } else {
      this.componentDidMountInNewMode();
    }
  }

  public render(): JSX.Element {
    const { id, className, styles, theme, titleOnPanels } = this.props;
    const {
      useName,
      labelTitleNew,
      labelTitleEdit,
      labelFormTemplateName,
      labelDialogTitleDelete,
      labelDialogSubtextPrefixDelete,
      labelDialogSubtextSuffixDelete,
      labelDialogBtnDelete,
      labelDialogBtnCancel,
      labelPanelSettingsPropertyTitle,
      labelPanelSettingsOperatorsTitle,
      labelPanelSettingsBtnUseAll,
      labelPanelSettingsBtnUseSelected,
      labelDialogTitleReset,
      labelDialogSubtextReset,
      labelDialogBtnReset,
      labelPanelDisplayPropertiesTitle,
      labelPanelAddPropertiesTitle,
      labelPanelMenusTitle,
      labelPanelAddBtnAdd,
      labelPanelAddBtnCancel,
      labelMovListAvailable,
      labelMovListSelected,
      labelMenuRemoveSelection,
      labelDialogTitleConfirmMainChange,
      labelDialogSubtextConfirmMainChange,
      labelDialogBtnConfirmMainChange,
      labelMainValidationMessage,
      labelPropertiesValidationMessage,
      labelDisplayPropertiesValidationMessage
    } = this.props;
    const {
      name,
      main,
      properties,
      showSettingsPanel,
      showMovablePanel,
      settingsOperatorsPropertyType,
      settingsOperatorsSelectionType,
      displayPropertiesPanel,
      addPropertiesPanel,
      menusPanel,
      displayProperties,
      movableMainValues,
      previewItems,
      previewLabel,
      fieldProperties,
      validateMainFieldsError,
      validatePropertiesFieldsError,
      validateDisplayPropertiesError
    } = this.state;
    const classNames = getClassNames(styles, { theme: theme!, className });
    const title = id ? `${labelTitleEdit} ${name}` : labelTitleNew;
    const propertiesLength = properties.length;

    let propertiesIdx = 0;

    return (
      <Fabric className={`ms-Grid ${classNames.root}`}>
        <div className="ms-Grid-row">
          <div className="ms-Grid-col ms-sm12">
            <CommandBar farItems={this.getFormCommandBarFarItems()} items={this.getFormCommandBarItems(id)} />
          </div>
        </div>
        <div className="ms-Grid-row tpl-a-f-nl-sm">
          <div className="ms-Grid-col ms-sm12" />
        </div>
        <div className="ms-Grid-row">
          <div className="ms-Grid-col ms-sm12">
            <Title title={title ? title : ''} icon={true} />
          </div>
        </div>
        {validateMainFieldsError && (
          <div className="ms-Grid-row" style={{ marginTop: '5px', marginBottom: '5px' }}>
            <div className="ms-Grid-col ms-sm12">
              <MessageBar
                messageBarType={MessageBarType.error}
                isMultiline={false}
                onDismiss={this.closeMainFieldsMessage}
                dismissButtonAriaLabel="Close"
              >
                {labelMainValidationMessage} {validateMainFieldsError}
              </MessageBar>
            </div>
          </div>
        )}
        {validatePropertiesFieldsError && (
          <div className="ms-Grid-row" style={{ marginTop: '5px', marginBottom: '5px' }}>
            <div className="ms-Grid-col ms-sm12">
              <MessageBar
                messageBarType={MessageBarType.error}
                isMultiline={false}
                onDismiss={this.closePropertiesFieldsMessage}
                dismissButtonAriaLabel="Close"
              >
                {labelPropertiesValidationMessage}
              </MessageBar>
            </div>
          </div>
        )}
        {validateDisplayPropertiesError && (
          <div className="ms-Grid-row" style={{ marginTop: '5px', marginBottom: '5px' }}>
            <div className="ms-Grid-col ms-sm12">
              <MessageBar
                messageBarType={MessageBarType.error}
                isMultiline={false}
                onDismiss={this.closeDisplayPropertiesMessage}
                dismissButtonAriaLabel="Close"
              >
                {labelDisplayPropertiesValidationMessage}
              </MessageBar>
            </div>
          </div>
        )}
        {useName && (
          <div className="ms-Grid-row">
            <div className="ms-Grid-col ms-sm12">
              <TextField
                key="TemplateName"
                value={name}
                label={labelFormTemplateName}
                required
                onBlur={e => {
                  this.result.name = e.target.value;
                  this.setState({ name: this.result.name });
                }}
              />
            </div>
          </div>
        )}
        {main.map(element => {
          let field = undefined;
          const rowKey = `MainRow_${element.id}`;
          const movablePanelKey = `MovablePanel_${element.id}`;
          const label = this.getDynamicTranslationInternal(element.name);
          const movableElements = movableMainValues.get(element.id)
            ? (movableMainValues.get(element.id) as IMovableOptionsListElement[])
            : element.values;
          const lastMovableElementsState = deepClone(movableElements);

          switch (element.field) {
            case FieldTypeEnum.INPUT: {
              const key = `MainTextField_${element.id}`;
              field = (
                <TextField
                  key={key}
                  onBlur={e => this.handleMainTextInput(e, element)}
                  value={this.getSafeValue(element.values)}
                  label={label}
                  required={element.required}
                />
              );
              break;
            }
            case FieldTypeEnum.LOOKUP: {
              const key = `MainDropdown_${element.id}`;
              field = (
                <Dropdown
                  multiSelect
                  key={key}
                  options={element.values}
                  selectedKeys={this.getSelectedKeys(element.values)}
                  label={label}
                  required={element.required}
                  onChange={(event, option?: IDropdownOption) => this.onMainChangeValues(event, element, option)}
                />
              );
              break;
            }
            case FieldTypeEnum.PICKUP: {
              const key = `MainPickup_${element.id}`;
              field = (
                <Dropdown
                  key={key}
                  options={element.values}
                  selectedKey={this.getSelectedKey(element.values)}
                  label={label}
                  required={element.required}
                  onChange={(event, option?: IDropdownOption | undefined, index?: number | undefined) =>
                    this.onMainChangeValuesPickup(event, element, option)
                  }
                />
              );
              break;
            }
            case FieldTypeEnum.DATEPICKER: {
              const key = `MainDatePicker_${element.id}`;
              field = (
                <DatePicker
                  key={key}
                  label={label}
                  onSelectDate={(date: Date | null | undefined) => this.handleMainDateInput(date, element)}
                />
              );
              break;
            }
            case FieldTypeEnum.MOVABLE: {
              const key = `MainMovable_${element.id}`;

              const value = this.getArrayAsString(movableElements);

              field = (
                <TextField
                  readOnly
                  key={key}
                  value={value}
                  label={label}
                  required={element.required}
                  onClick={() => this.onShowMovablePanel(element, lastMovableElementsState)}
                />
              );
              break;
            }
            default: {
              break;
            }
          }
          return (
            <div className="ms-Grid-row" key={rowKey}>
              <div className="ms-Grid-col ms-sm12">{field}</div>
              <div className="ms-Grid-row tpl-a-f-nl-md">
                <div className="ms-Grid-col ms-sm12">
                  {!this.state.hideConfirmMainChange && (
                    <Dialog
                      hidden={this.state.hideConfirmMainChange}
                      onDismiss={() => this.restoreLastMainState()}
                      dialogContentProps={{
                        type: DialogType.normal,
                        title: `${labelDialogTitleConfirmMainChange}`,
                        subText: `${labelDialogSubtextConfirmMainChange}`
                      }}
                      modalProps={{
                        titleAriaId: `confirmMainChangeLabelId`,
                        subtitleAriaId: `confirmMainChangeSubTextId`,
                        isBlocking: true,
                        containerClassName: 'ms-dialogMainOverride'
                      }}
                    >
                      <DialogFooter>
                        <PrimaryButton onClick={() => this.resetValuesDueToMainchange()} text={labelDialogBtnConfirmMainChange} />
                        <DefaultButton onClick={() => this.restoreLastMainState()} text={labelDialogBtnCancel} />
                      </DialogFooter>
                    </Dialog>
                  )}
                  {showMovablePanel[element.id] && (
                    <Panel
                      key={movablePanelKey}
                      isOpen={showMovablePanel[element.id]}
                      type={PanelType.medium}
                      onDismiss={() => this.onDismissMovablePanel(element, lastMovableElementsState)}
                      closeButtonAriaLabel="Close"
                      headerText={titleOnPanels ? '' : element.name}
                    >
                      <div className="ms-Grid">
                        <div className="ms-Grid-row">
                          <div className="ms-Grid-col ms-sm12" style={{ height: '500px' }}>
                            {titleOnPanels && (
                              <MovableOptionsList
                                elements={movableElements}
                                height={'350px'}
                                title={element.name}
                                disableUpAndDown={true}
                                availableLabel={labelMovListAvailable}
                                selectedLabel={labelMovListSelected}
                              />
                            )}
                            {!titleOnPanels && (
                              <MovableOptionsList
                                elements={
                                  movableMainValues.get(element.id)
                                    ? (movableMainValues.get(element.id) as IMovableOptionsListElement[])
                                    : element.values
                                }
                                height={'350px'}
                                disableUpAndDown={true}
                              />
                            )}
                          </div>
                        </div>
                      </div>
                    </Panel>
                  )}
                </div>
              </div>
            </div>
          );
        })}
        <div className="ms-Grid-row tpl-f-nl-sm">
          <div className="ms-Grid-col ms-sm12" />
        </div>
        {properties.map(element => {
          let field = undefined;
          const rowKey = `PropertiesRow_${element.id}`;
          const fieldName = element.name;
          const operationsKey = `PropertiesDropdownOperations_${element.id}`;
          const typesKey = `PropertiesDropdownTypes_${element.id}`;
          const panelSettingsKey = `PropertiesPanelSettings_${element.id}`;
          const panelSettingsTitle = `${element.name}`;
          const operatorsChoiceGroupKey = `PropertieOperatorsChoiceGroup_${element.id}`;
          const elementOperatorProperties: IDropdownOption[] = this.getElementOperatorProperties(element.id);

          switch (element.field) {
            case FieldTypeEnum.INPUT: {
              const key = `PropertiesTextField_${element.id}`;
              field = (
                <TextField key={key} value={this.getSafeValue(element.values)} onBlur={e => this.handlePropertiesTextInput(e, element)} />
              );
              break;
            }
            case FieldTypeEnum.LOOKUP: {
              const key = `PropertiesDropdown_${element.id}`;
              field = (
                <Dropdown
                  multiSelect
                  key={key}
                  options={element.values}
                  onChange={(event: React.FormEvent<HTMLDivElement>, option?: IDropdownOption | undefined, index?: number | undefined) =>
                    this.onPropertiesChangeValues(event, element, option)
                  }
                />
              );
              break;
            }
            case FieldTypeEnum.PICKUP: {
              const key = `PropertiesPickup_${element.id}`;
              field = (
                <Dropdown
                  key={key}
                  options={element.values}
                  onChange={(event: React.FormEvent<HTMLDivElement>, option?: IDropdownOption | undefined, index?: number | undefined) =>
                    this.onPropertiesChangeValuesPickup(event, element, option)
                  }
                />
              );
              break;
            }
            case FieldTypeEnum.DATEPICKER: {
              const key = `PropertiesDatePicker_${element.id}`;
              field = (
                <DatePicker
                  key={key}
                  value={this.getSafeDateValue(element.values)}
                  onSelectDate={(date: Date | null | undefined) => this.handlePropertiesDateInput(date, element)}
                />
              );
              break;
            }
            default: {
              break;
            }
          }
          return (
            <div key={rowKey} style={{ marginBottom: 10 }}>
              <Stack horizontal verticalFill wrap horizontalAlign="space-between" verticalAlign="center">
                <Stack.Item grow={1}>
                  <Label className="ms-font-m-plus">{fieldName}:</Label>
                </Stack.Item>
                <Stack.Item grow={0} styles={{ root: { minWidth: 160 } }}>
                  <CommandBar
                    items={this.getOperatorActionItems(element, propertiesIdx++, propertiesLength)}
                    styles={{ root: { backgroundColor: '#fff', padding: 0 } }}
                  />
                </Stack.Item>
              </Stack>
              <Stack horizontal wrap tokens={{ childrenGap: 5 }}>
                <Stack.Item grow={0} disableShrink styles={{ root: { width: 180 } }}>
                  <Dropdown
                    key={operationsKey}
                    options={element.operations}
                    onChange={(event: React.FormEvent<HTMLDivElement>, option?: IDropdownOption | undefined, index?: number | undefined) =>
                      this.onChangePropertyOperation(event, element, option)
                    }
                  />
                </Stack.Item>
                <Stack.Item grow={2}>{field}</Stack.Item>
              </Stack>

              <div>
                {!this.state.hideDeleteDialogs[element.id] && (
                  <Dialog
                    hidden={this.state.hideDeleteDialogs[element.id]}
                    onDismiss={() => this.closeDeleteDialog(element.id)}
                    dialogContentProps={{
                      type: DialogType.normal,
                      title: `${labelDialogTitleDelete} [${element.name}]`,
                      subText: `${labelDialogSubtextPrefixDelete} [${element.name}] ${labelDialogSubtextSuffixDelete}`
                    }}
                    modalProps={{
                      titleAriaId: `deletePropertyLabelId${element.id}`,
                      subtitleAriaId: `deletePropertySubTextId${element.id}`,
                      isBlocking: false,
                      containerClassName: 'ms-dialogMainOverride'
                    }}
                  >
                    <DialogFooter>
                      <PrimaryButton onClick={() => this.removeProperty(element.id)} text={labelDialogBtnDelete} />
                      <DefaultButton onClick={() => this.closeDeleteDialog(element.id)} text={labelDialogBtnCancel} />
                    </DialogFooter>
                  </Dialog>
                )}
                {showSettingsPanel[element.id] && (
                  <Panel
                    key={panelSettingsKey}
                    isOpen={showSettingsPanel[element.id]}
                    type={PanelType.smallFixedFar}
                    onDismiss={() => this.onDismissSettingsPanel(element)}
                    closeButtonAriaLabel="Close"
                    headerText={titleOnPanels ? '' : panelSettingsTitle}
                  >
                    <div className="ms-Grid">
                      {titleOnPanels && (
                        <div className="ms-Grid-row">
                          <div className="ms-Grid-col ms-sm12">
                            <Title title={panelSettingsTitle} icon={true} />
                          </div>
                        </div>
                      )}
                      <div className="ms-Grid-row">
                        <div className="ms-Grid-col ms-sm5 ms-md5 ms-lg5">
                          <Label className="ms-font-m-plus">{labelPanelSettingsPropertyTitle}</Label>
                        </div>
                        <div className="ms-Grid-col ms-sm7 ms-md7 ms-lg7">
                          <Dropdown
                            key={typesKey}
                            options={fieldProperties}
                            onChange={(event: React.FormEvent<HTMLDivElement>, option?: IDropdownOption) =>
                              this.onChangePropertyType(event, option, element)
                            }
                            selectedKey={String(settingsOperatorsPropertyType.get(element.id))}
                          />
                        </div>
                      </div>
                      <div className="ms-Grid-row tpl-a-f-nl-x-sm">
                        <div className="ms-Grid-col ms-sm12" />
                      </div>
                      <div className="ms-Grid-row">
                        <div className="ms-Grid-col ms-sm5 ms-md5 ms-lg5">
                          <Label className="ms-font-m-plus">{labelPanelSettingsOperatorsTitle}</Label>
                        </div>
                        <div className="ms-Grid-col ms-sm7 ms-md7 ms-lg7">
                          <ChoiceGroup
                            key={operatorsChoiceGroupKey}
                            onChange={(
                              ev?: React.FormEvent<HTMLElement | HTMLInputElement> | undefined,
                              option?: IChoiceGroupOption | undefined
                            ) => this.onChangeSettingsOperatorType(ev, option, element)}
                            defaultSelectedKey={
                              settingsOperatorsSelectionType[element.id]
                                ? settingsOperatorsSelectionType[element.id]
                                : OperatorsSettingsTypeEnum.USE_ALL
                            }
                            options={[
                              {
                                key: OperatorsSettingsTypeEnum.USE_ALL,
                                text: labelPanelSettingsBtnUseAll ? labelPanelSettingsBtnUseAll : ''
                              },
                              {
                                key: OperatorsSettingsTypeEnum.USE_SELECTED,
                                text: labelPanelSettingsBtnUseSelected ? labelPanelSettingsBtnUseSelected : ''
                              }
                            ]}
                            required={true}
                          />
                        </div>
                      </div>
                      <div className="ms-Grid-row tpl-a-f-nl-x-sm">
                        <div className="ms-Grid-col ms-sm12" />
                      </div>
                      <div className="ms-Grid-row">
                        <div className="ms-Grid-col ms-sm5 ms-md5 ms-lg5" />
                        <div className="ms-Grid-col ms-sm7 ms-md7 ms-lg7">
                          <div className="ms-Grid">
                            {elementOperatorProperties.map(operator => {
                              const operatorKey = Number(operator.key);
                              return (
                                <div key={`DivOperatorKey${operatorKey}`} className="ms-Grid-row tpl-a-f-nl-sm-c">
                                  <div className="ms-Grid-col ms-sm12">
                                    <Checkbox
                                      key={`OperatorKey${operatorKey}`}
                                      label={operator.text}
                                      checked={this.isSettingsOperatorChecked(element.id, operatorKey)}
                                      onChange={(
                                        ev?: React.FormEvent<HTMLElement | HTMLInputElement> | undefined,
                                        checked?: boolean | undefined
                                      ) => this.onChangeSettingsOperators(element.id, operatorKey, ev, checked)}
                                    />
                                  </div>
                                </div>
                              );
                            })}
                          </div>
                        </div>
                      </div>
                    </div>
                  </Panel>
                )}
              </div>
            </div>
          );
        })}
        {!this.state.hideResetDialog && (
          <Dialog
            hidden={this.state.hideResetDialog}
            onDismiss={this.closeResetDialog}
            dialogContentProps={{
              type: DialogType.normal,
              title: labelDialogTitleReset,
              subText: labelDialogSubtextReset
            }}
            modalProps={{
              titleAriaId: 'resetLabelId',
              subtitleAriaId: 'resetSubTextId',
              isBlocking: false,
              containerClassName: 'ms-dialogMainOverride'
            }}
          >
            <DialogFooter>
              <PrimaryButton onClick={() => this.reset()} text={labelDialogBtnReset} />
              <DefaultButton onClick={this.closeResetDialog} text={labelDialogBtnCancel} />
            </DialogFooter>
          </Dialog>
        )}
        {displayPropertiesPanel && (
          <Panel
            key="DisplayPropertiesPanelKey"
            isOpen={displayPropertiesPanel}
            type={PanelType.medium}
            onDismiss={() => this.onDismissDisplayPropertiesPanel()}
            closeButtonAriaLabel="Close"
            headerText={titleOnPanels ? '' : labelPanelDisplayPropertiesTitle}
          >
            <div className="ms-Grid">
              <div className="ms-Grid-row">
                <div className="ms-Grid-col ms-sm12" style={{ height: '500px' }}>
                  {titleOnPanels && (
                    <MovableOptionsList
                      elements={displayProperties}
                      height={'350px'}
                      title={'Display Properties'}
                      handleMoveUpAndDownChange={this.onUpDownDisplayProperties}
                    />
                  )}
                  {!titleOnPanels && (
                    <MovableOptionsList
                      elements={displayProperties}
                      height={'350px'}
                      handleMoveUpAndDownChange={this.onUpDownDisplayProperties}
                    />
                  )}
                </div>
              </div>
            </div>
          </Panel>
        )}
        {addPropertiesPanel && (
          <Panel
            key="AddPropertiesPanelKey"
            isOpen={addPropertiesPanel}
            type={PanelType.medium}
            onDismiss={() => this.onShowAddPropertiesPanel(false)}
            closeButtonAriaLabel="Close"
            headerText={titleOnPanels ? '' : labelPanelAddPropertiesTitle}
          >
            <div className="ms-Grid">
              {titleOnPanels && (
                <div className="ms-Grid-row">
                  <div className="ms-Grid-col ms-sm12">
                    <Title title={labelPanelAddPropertiesTitle ? labelPanelAddPropertiesTitle : ''} icon={true} />
                  </div>
                </div>
              )}
              <div className="ms-Grid-row">
                <div className="ms-Grid-col ms-sm12" style={{ height: '300px' }}>
                  <List
                    getUrl={this.getPropertiesUrl}
                    handleResponse={this.handleGetPropertiesResponse}
                    onCustomBuildColumns={this.onBuildPropertiesColumns}
                    enableSearch={true}
                    enableSort={true}
                    selection={this.selection}
                    selectionMode={SelectionMode.multiple}
                    height={'250px'}
                  />
                </div>
              </div>
              <div className="ms-Grid-row tpl-a-f-nl-sm">
                <div className="ms-Grid-col ms-sm12" />
              </div>
              <div className="ms-Grid-row">
                <div className="ms-Grid-col ms-sm8 ms-md8 ms-lg8" />
                <div className="ms-Grid-col ms-sm2 ms-md2 ms-lg2">
                  <ActionButton iconProps={{ iconName: 'Add' }} onClick={() => this.onAddProperties()}>
                    {labelPanelAddBtnAdd}
                  </ActionButton>
                </div>
                <div className="ms-Grid-col ms-sm2 ms-md2 ms-lg2">
                  <ActionButton iconProps={{ iconName: 'Cancel' }} onClick={() => this.onShowAddPropertiesPanel(false)}>
                    {labelPanelAddBtnCancel}
                  </ActionButton>
                </div>
              </div>
            </div>
          </Panel>
        )}
        {menusPanel && (
          <Panel
            key="MenusPanelKey"
            isOpen={menusPanel}
            type={PanelType.medium}
            onDismiss={() => this.onShowMenusPanel(false)}
            closeButtonAriaLabel="Close"
            headerText={titleOnPanels ? '' : labelPanelMenusTitle}
          >
            <div className="ms-Grid">
              {titleOnPanels && (
                <div className="ms-Grid-row">
                  <div className="ms-Grid-col ms-sm12">
                    <Title title={labelPanelMenusTitle ? labelPanelMenusTitle : ''} icon={true} />
                  </div>
                </div>
              )}
              <div className="ms-Grid-row">
                <div className="ms-Grid-col ms-sm12">
                  <fieldset style={{ borderWidth: 'thin', borderColor: '#fff' }}>
                    <legend>{previewLabel}</legend>
                    <CommandBar items={previewItems} />
                  </fieldset>
                </div>
              </div>
              <div className="ms-Grid-row" style={{ paddingTop: '30px' }}>
                <div className="ms-Grid-col ms-sm12" style={{ height: '300px' }}>
                  <List
                    getUrl={this.getMenusUrl}
                    handleResponse={this.handleGetMenusResponse}
                    onCustomBuildColumns={this.onBuildMenusColumns}
                    enableSearch={true}
                    enableSort={true}
                    selection={this.menusSelection}
                    selectionMode={SelectionMode.single}
                    height={'250px'}
                  />
                </div>
              </div>
              <div className="ms-Grid-row">
                <div className="ms-Grid-col ms-sm8 ms-md8 ms-lg8" />
                <div className="ms-Grid-col ms-sm4 ms-md4 ms-lg4" style={{ textAlign: 'right' }}>
                  <ActionButton iconProps={{ iconName: 'Delete' }} onClick={this.unLinkMenu}>
                    {labelMenuRemoveSelection}
                  </ActionButton>
                </div>
              </div>
            </div>
          </Panel>
        )}
      </Fabric>
    );
  }

  private getElementOperatorProperties = (id: any): IDropdownOption[] => {
    const { operatorsProperties } = this.state;
    let elementOperatorProperties: IDropdownOption[] | undefined = operatorsProperties.get(id);
    if (!elementOperatorProperties) {
      elementOperatorProperties = [];
    }
    return elementOperatorProperties;
  };

  private unLinkMenu = (): void => {
    this.result.menuId = '';
    this.setState({ previewItems: [], previewLabel: this.props.labelFormFieldset });
  };

  private checkIfCanOpenDisplayProperties = (): boolean => {
    return this.canLoadDisplayProperties;
  };

  private resetValuesDueToMainchange = (): void => {
    const { movableMainValues } = this.state;
    this.result.properties.clear();
    this.canLoadDisplayProperties = true;
    this.result.displayProperties.splice(0, this.result.displayProperties.length);

    if (this.canResetMovableResult) {
      movableMainValues.forEach((values: IMovableOptionsListElement[], key: number) => {
        const row = this.result.main.get(key);
        if (row) {
          row.values.splice(0, row.values.length);
          this.result.main.set(key, row);
        }
      });
    }

    this.setState({
      properties: [],
      displayProperties: [],
      hideConfirmMainChange: true
    });
  };

  private loadDisplayProperties = (show: boolean): void => {
    if (show) {
      if (!this.checkIfCanOpenDisplayProperties()) {
        console.log('No need to load Display Properties');
        this.setState({ displayPropertiesPanel: true });
        return;
      }
    }

    const { id } = this.props;
    const mainBodyValues = this.getEncodedMainValues('?');
    const url = this.mode === FormModeEnum.EDIT ? `${this.url}/${id}/display/${mainBodyValues}` : `${this.url}/display/${mainBodyValues}`;

    console.log(url);

    Axios.get(url)
      .then(response => {
        const displayDto = response.data;
        this.setDisplayProperties(displayDto.data);
        this.initialState.displayProperties = deepClone(displayDto.data);
        if (show) {
          this.setState({ displayPropertiesPanel: true }, () => (this.canLoadDisplayProperties = false));
        } else {
          this.canLoadDisplayProperties = false;
          this.onDismissDisplayPropertiesPanel();
        }
      })
      .catch(error => console.log(error));
  };

  private componentDidMountInEditMode() {
    const { id, allowDisplayProperties, allowMenus } = this.props;
    const mainUrl = `${this.url}/${id}/main`;
    const propertiesUrl = `${this.url}/${id}/properties`;

    Axios.get(mainUrl)
      .then(response => {
        const mainDto = response.data;
        this.setState({
          name: mainDto.name,
          main: mainDto.data
        });
        this.resetResultMainValues(id, mainDto.name, mainDto.data);
        this.initialState.name = mainDto.name;
        this.initialState.main = deepClone(mainDto.data);
      })
      .catch(error => this.setState({ error: error }));

    Axios.get(propertiesUrl)
      .then(response => {
        const propertiesDto = response.data;
        this.setState({
          properties: propertiesDto.data
        });
        this.setResultPropertiesValues(propertiesDto.data);
        this.initialState.properties = deepClone(propertiesDto.data);
      })
      .catch(error => this.setState({ error: error }));

    if (allowMenus) {
      this.loadMenuData();
    }

    this.loadFieldProperties();

    if (allowDisplayProperties) {
      this.loadDisplayProperties(false);
    }
  }

  private componentDidMountInNewMode() {
    const mainUrl = `${this.url}/main`;
    const nullId = -1;

    Axios.get(mainUrl)
      .then(response => {
        const mainDto = response.data;
        this.setState({
          main: mainDto.data
        });
        this.resetResultMainValues(nullId, '', mainDto.data);
        this.initialState.main = deepClone(mainDto.data);
      })
      .catch(error => this.setState({ error: error }));

    this.loadFieldProperties();
  }

  private loadMenuData = (): void => {
    const { id } = this.props;
    const menuUrl = `${this.url}/${id}/menu`;

    Axios.get(menuUrl)
      .then(response => {
        const menuDto = response.data;
        this.result.menuId = menuDto.id;
        this.populatePreviewItems(menuDto);
      })
      .catch(error => console.log(error));
  };

  private loadFieldProperties = (): void => {
    const { baseUrl } = this.props;
    const url = `${baseUrl}/properties`;

    Axios.get(url)
      .then(response => {
        const dto = response.data;
        this.setState({ fieldProperties: dto.data });
        this.initialState.displayProperties = deepClone(dto.data);
      })
      .catch(error => console.log(error));
  };

  private loadOperatorProperties = (element: any): void => {
    const { baseUrl } = this.props;
    const { operatorsProperties } = this.state;
    const url = `${baseUrl}/properties/operators?id=${element.id}&name=${element.name}&isSystemField=${element.isSystemField}&type=${
      element.type
    }&field=${element.field}`;

    console.log(url);

    Axios.get(url)
      .then(response => {
        const dto = response.data;

        const newOperatorProperties = cloneMap(operatorsProperties);
        newOperatorProperties.set(element.id, dto);

        this.setState({ operatorsProperties: newOperatorProperties });

        const id = element.id;
        const { showSettingsPanel } = this.state;
        const newShowSettingsPanel = update(showSettingsPanel, { [id]: { $set: true } });
        this.setState({ showSettingsPanel: newShowSettingsPanel });
      })
      .catch(error => console.log(error));
  };

  private getSafeValue = (values: any[]): any => {
    return values != null && values.length ? values[0].text : '';
  };

  private getSafeDateValue = (values: any[]): any => {
    return values != null && values.length ? this.getSafeTextToDate(values[0].text) : null;
  };

  private getSafeTextToDate = (value: any): any => {
    if (!value) {
      return null;
    }

    let safeDate = null;
    try {
      const asNumber: number = Number(value);
      safeDate = new Date(0);
      safeDate.setUTCSeconds(asNumber);
    } catch (e) {
      console.log('Error converting text to date:', value);
    }

    return safeDate;
  };

  private removeProperty = (id: number): void => {
    const { properties } = this.state;
    const indexToRemove = properties.findIndex(p => p.id === id);

    if (indexToRemove >= 0) {
      const newProperties = update(properties, { $splice: [[indexToRemove, 1]] });
      this.setResultPropertiesValues(newProperties);
      this.setState({ properties: newProperties }, () => this.closeDeleteDialog(id));
    } else {
      this.closeDeleteDialog(id);
    }
  };

  private movePropertyUp = (id: number): void => {
    this.moveProperty(id, -1);
  };

  private movePropertyDown = (id: number): void => {
    this.moveProperty(id, 1);
  };

  private moveProperty = (id: number, pos: number): void => {
    const { properties } = this.state;
    const index = properties.findIndex(p => p.id === id);

    if (index >= 0) {
      const element = properties[index];
      let newProperties = update(properties, { $splice: [[index, 1]] });
      newProperties = update(newProperties, { $splice: [[index + pos, 0, element]] });
      this.setResultPropertiesValues(newProperties);
      this.setState({ properties: newProperties });
    }
  };

  private getArrayAsString = (values: IMovableOptionsListElement[]): string => {
    return values != null
      ? values
          .filter((value: any) => value.selected === true)
          .map((item: any) => item.text)
          .join(', ')
      : '';
  };

  private onDismissMovablePanel = (element: any, lastMovableElements: IMovableOptionsListElement[]): void => {
    const { showMovablePanel, movableMainValues } = this.state;
    const newShowMovablePanel = update(showMovablePanel, { [element.id]: { $set: false } });
    const values: IFieldValue[] = [];
    const elementValues = movableMainValues.get(element.id) ? movableMainValues.get(element.id) : element.values;

    let numberOfNotSelectedElements = 0;

    const valuesHaveNotChanged =
      elementValues.length == lastMovableElements.length &&
      elementValues.every(function(element: IMovableOptionsListElement, index: number) {
        return element.selected === lastMovableElements[index].selected;
      });

    for (let lastElement of lastMovableElements) {
      if (!lastElement.selected) {
        numberOfNotSelectedElements++;
      }
    }

    elementValues.forEach((elem: any) => {
      if (elem.selected) {
        const fieldValue: IFieldValue = {
          id: elem.key,
          value: elem.text
        };
        values.push(fieldValue);
      }
    });

    const formRow = this.getMainRowFromId(element.id, element.name);
    const updatedSelectedItems = update(formRow.values, { $set: values });
    formRow.values = updatedSelectedItems;

    this.result.main.set(element.id, formRow);

    const noElementWasSelected = numberOfNotSelectedElements === lastMovableElements.length;

    if (!elementValues || valuesHaveNotChanged || noElementWasSelected) {
      this.setState({ showMovablePanel: newShowMovablePanel });
      return;
    }

    this.setState({ showMovablePanel: newShowMovablePanel }, () => this.openConfirmMainChange());
  };

  private onShowMovablePanel = (element: any, lastMovableElements: IMovableOptionsListElement[]): void => {
    const id = element.id;
    const lastValues: IFieldValue[] = [];

    this.canResetMovableResult = false;

    lastMovableElements.map(element => {
      const value: IFieldValue = {
        id: element.key,
        value: element.text,
        selected: element.selected
      };
      lastValues.push(value);
    });

    const { movableMainValues } = this.state;
    const newMovableMainValues = cloneMap(movableMainValues);
    newMovableMainValues.set(id, lastMovableElements);
    this.setState({ movableMainValues: newMovableMainValues }, () => this.storeLastMainState());

    const { showMovablePanel } = this.state;
    const newShowMovablePanel = update(showMovablePanel, { [id]: { $set: true } });
    this.setState({ showMovablePanel: newShowMovablePanel });
  };

  private onShowSettingsPanel = (element: any): void => {
    this.loadOperatorProperties(element);
  };

  private onDismissSettingsPanel = (element: any): void => {
    const id = element.id;
    const { showSettingsPanel, properties, settingsOperators } = this.state;
    const operators = settingsOperators.get(id);
    const newOperations: any = [];
    let firstKey: any = undefined;

    const elementOperatorProperties: IDropdownOption[] = this.getElementOperatorProperties(element.id);

    if (operators) {
      operators.forEach((selected: boolean, key: number) => {
        const index = elementOperatorProperties.findIndex(p => String(p.key) === String(key));
        const obj = elementOperatorProperties[index];
        if (obj && selected) {
          if (!firstKey) {
            firstKey = key;
          }
          const value = {
            key: key,
            text: obj.text,
            selected: firstKey ? true : false
          };
          newOperations.splice(key, 0, value);
        }
      });
    }

    const index = properties.findIndex(p => p.id === id);
    const newProperties = update(properties, { [index]: { operations: { $set: newOperations } } });
    const newShowSettingsPanel = update(showSettingsPanel, { [id]: { $set: false } });
    this.setResultPropertiesValues(newProperties);
    this.setState({ properties: newProperties, showSettingsPanel: newShowSettingsPanel });
  };

  private onChangePropertyOperation = (event: React.FormEvent<HTMLDivElement>, element: any, item: IDropdownOption | undefined): void => {
    if (!item) {
      return;
    }
    const id: number = element.id;
    const fieldName: string = element.name;
    const formRow = this.getPropertiesRowFromId(id, fieldName, element.isSystemField);
    formRow.selectedOperation = Number(item.key);
    this.result.properties.set(id, formRow);
  };

  private onChangeSettingsOperators = (
    id: number,
    index: number,
    ev?: React.FormEvent<HTMLElement | HTMLInputElement> | undefined,
    isChecked?: boolean | undefined
  ): void => {
    const { settingsOperators } = this.state;
    const operatorsArray = settingsOperators.get(id) ? settingsOperators.get(id) : [];
    const newOperatorsArray = update(operatorsArray, { [index]: { $set: isChecked! } });
    const newSettingsOperators = cloneMap(settingsOperators);
    newSettingsOperators.set(id, newOperatorsArray as any[]);

    this.setState({ settingsOperators: newSettingsOperators });
  };

  private isSettingsOperatorChecked = (id: number, index: number): boolean => {
    const { settingsOperators } = this.state;
    const operatorsArray = settingsOperators.get(id) ? settingsOperators.get(id) : [];
    const result = operatorsArray ? operatorsArray[index] : false;

    return result ? result : false;
  };

  private onChangeSettingsOperatorType = (
    ev: React.FormEvent<HTMLElement | HTMLInputElement> | undefined,
    option: any,
    element: any
  ): void => {
    const { settingsOperators, settingsOperatorsSelectionType } = this.state;

    const id = element.id;

    const newSettingsOperatorsSelectionType = update(settingsOperatorsSelectionType, { [id]: { $set: option.key } });

    let newSettingsOperatorsValues: boolean[] = [];

    const isSelectOption = OperatorsSettingsTypeEnum.USE_ALL === option.key ? true : false;

    const elementOperatorProperties: IDropdownOption[] = this.getElementOperatorProperties(element.id);

    for (let operator of elementOperatorProperties) {
      newSettingsOperatorsValues[operator.key] = isSelectOption;
    }

    const newSettingsOperators = cloneMap(settingsOperators);
    newSettingsOperators.set(id, newSettingsOperatorsValues);

    this.setState({
      settingsOperatorsSelectionType: newSettingsOperatorsSelectionType,
      settingsOperators: newSettingsOperators
    });
  };

  private onShowDisplayPropertiesPanel = () => {
    if (this.validateMainFields(false)) {
      this.loadDisplayProperties(true);
    }
  };

  private processDisplayProperties = (elements: IMovableOptionsListElement[]): IMovableOptionsListElement[] => {
    const newDisplayProperties: any[] = [];
    for (let property of elements) {
      if (property.selected) {
        newDisplayProperties.push(property);
      }
    }
    return newDisplayProperties;
  };

  private onDismissDisplayPropertiesPanel = () => {
    const { displayProperties } = this.state;
    const newDisplayProperties = this.processDisplayProperties(displayProperties);
    this.result.displayProperties = newDisplayProperties;
    this.setState({ displayPropertiesPanel: false });
  };

  private onUpDownDisplayProperties = (elements: IMovableOptionsListElement[]) => {
    const { displayProperties } = this.state;
    const newDisplayProperties = this.processDisplayProperties(elements);
    const temp: IMovableOptionsListElement[] = [];

    displayProperties.forEach(element => {
      const index = newDisplayProperties.findIndex(p => p.key === element.key);
      if (index < 0) {
        const newElement: IMovableOptionsListElement = {
          key: element.key,
          text: element.text,
          selected: false
        };
        temp.push(newElement);
      }
    });
    newDisplayProperties.push(...temp);
    this.setState({ displayProperties: newDisplayProperties });
  };

  private onShowAddPropertiesPanel = (show: boolean) => {
    if (this.validateMainFields(false)) {
      this.setState({ addPropertiesPanel: show });
    }
  };

  private onShowMenusPanel = (show: boolean) => {
    this.setState({ menusPanel: show });
  };

  private handleGetPropertiesResponse = (items: any): any => {
    return items;
  };

  private handleGetMenusResponse = (items: any[]): any[] => {
    return items;
  };

  private onDisplayPropertiesSelectionChanged = (selection: any[]): any => {
    const { properties } = this.state;
    const newPropertiesTemp = [];

    for (let element of selection) {
      const newElement = { ...element };
      const indexProperties = properties.findIndex(p => p.id === element.id);
      if (indexProperties < 0) {
        newPropertiesTemp.push(newElement);
      }
    }
    this.setState({ propertiesTemp: newPropertiesTemp });
  };

  private onMenusSelectionChanged = (selection: any[]): void => {
    const menu = selection[0];
    if (menu) {
      this.result.menuId = menu.id;
      this.populatePreviewItems(menu);
    }
  };

  private populatePreviewItems = (menuDto: any): void => {
    const { labelFormFieldset } = this.props;
    const previewCmdbarItems: any[] = [];
    const previewName: string = `${labelFormFieldset}: [ ${menuDto.name} ]`;

    const items = menuDto.items;

    items.forEach((element: any) => {
      const item = {
        key: element.id,
        name: element.name,
        iconProps: {
          iconName: element.icon
        },
        iconOnly: !element.textMode
      };
      previewCmdbarItems.push(item);
    });

    this.setState({ previewLabel: previewName, previewItems: previewCmdbarItems });
  };

  private isNull = (value: any): boolean => {
    return value == null || value === 'null' || value == undefined;
  };

  private getPropertiesUrl = (search: string, page: number, sortBy?: string, sortDesc?: boolean): string => {
    const { id } = this.props;
    const { properties } = this.state;
    const qValue = this.isNull(search) ? '' : `&q=${search}`;
    const sortByValue = this.isNull(sortBy) ? '' : `&_sort=${sortBy}`;
    const urlSuffix = id ? `/${id}/properties/add` : '/properties/add';
    let sortDescValue = '';

    if (!this.isNull(sortDesc)) {
      sortDescValue = sortDesc ? '&_order=desc' : '&_order=asc';
    }

    const propertiesDetails: IFieldValue[] = [];

    for (let property of properties) {
      const detail: IFieldValue = {
        id: property.id,
        value: property.name
      };
      propertiesDetails.push(detail);
    }

    const propertiesDetailsEncodedValues = encodeURIComponent(JSON.stringify({ details: propertiesDetails }));
    const propertiesValues =
      propertiesDetailsEncodedValues && propertiesDetailsEncodedValues.length > 0 ? `&properties=${propertiesDetailsEncodedValues}` : '';

    const mainBodyValues = this.getEncodedMainValues('&');
    const url = `${
      this.url
    }${urlSuffix}?_start=${page}&_limit=20${qValue}${sortByValue}${sortDescValue}${mainBodyValues}${propertiesValues}`;

    console.log(url);
    return url;
  };

  private getEncodedMainValues = (separator: string): string => {
    const main: ITemplateAdminMainRow[] = [];

    this.result.main.forEach((value: ITemplateAdminMainRow, key: number) => {
      main.push(value);
    });

    const mainValues = encodeURIComponent(JSON.stringify({ main: main }));
    return mainValues && mainValues.length > 0 ? `${separator}main=${mainValues}` : '';
  };

  private onAddProperties = (): void => {
    const { propertiesTemp, properties, id } = this.state;
    let propertiesUrl = id ? `${this.url}/${id}/properties/details?` : `${this.url}/properties/details?`;

    const details: IFieldValue[] = [];

    for (let property of propertiesTemp) {
      const detail: IFieldValue = {
        id: property.key,
        value: property.text
      };
      details.push(detail);
    }

    const encodedDetails = encodeURIComponent(JSON.stringify({ details: details }));
    const detailValues = encodedDetails && encodedDetails.length > 0 ? `&properties=${encodedDetails}` : '';

    propertiesUrl = propertiesUrl + this.getEncodedMainValues('&') + detailValues;
    console.log(propertiesUrl);

    Axios.get(propertiesUrl).then(response => {
      if (response.data.data.length > 0) {
        const values = response.data.data;
        const newProperties = update(properties, { $push: values });
        this.setResultPropertiesValues(newProperties);
        this.setState({ properties: newProperties, propertiesTemp: [] }, () => this.onShowAddPropertiesPanel(false));
      } else {
        this.onShowAddPropertiesPanel(false);
      }
    });
  };

  private getMenusUrl = (search: string, page: number, sortBy?: string, sortDesc?: boolean): string => {
    const { baseUrl } = this.props;
    const qValue = this.isNull(search) ? '' : `&q=${search}`;
    const sortByValue = this.isNull(sortBy) ? '' : `&_sort=${sortBy}`;
    let sortDescValue = '';
    let idsUriValues = '';

    if (!this.isNull(sortDesc)) {
      sortDescValue = sortDesc ? '&_order=desc' : '&_order=asc';
    }
    const url = `${baseUrl}/menus?_start=${page}&_limit=20${qValue}${sortByValue}${sortDescValue}${idsUriValues}`;
    console.log(url);
    return url;
  };

  private reset = (): void => {
    this.setState(this.emptyState, () => this.resetState());
  };

  private setMainSingleValue(id: number, fieldName: string, value: any): void {
    const { main } = this.state;
    const formRow = this.getMainRowFromId(id, fieldName);
    const currentValue: string = formRow.values != null && formRow.values.length ? String(formRow.values[0].value) : '';

    if (currentValue == String(value).trim()) {
      return;
    }

    //this.storeLastMainState(id, main, formRow);

    const fieldValue: IFieldValue = {
      id: '',
      value: value
    };
    formRow.values[0] = fieldValue;

    this.result.main.set(id, formRow);

    const index = main.findIndex(p => p.id === id);
    const newMain = update(main, { [index]: { values: { $set: [{ text: value }] } } });
    //this.setState({ main: newMain }, () => this.openConfirmMainChange(id));
    this.setState({ main: newMain });
  }

  private cloneMovableOptions = (map: Map<number, IMovableOptionsListElement[]>): Map<number, IMovableOptionsListElement[]> => {
    const clonedMap = new Map<number, IMovableOptionsListElement[]>();
    map.forEach((values: IMovableOptionsListElement[], key: number) => {
      const clonedArray: IMovableOptionsListElement[] = [];
      values.forEach(value => {
        const element = {
          key: value.key,
          text: value.text,
          selected: value.selected
        };
        clonedArray.push(element);
      });
      clonedMap.set(key, clonedArray);
    });
    return clonedMap;
  };

  private storeLastMainState = (): void => {
    const { main, movableMainValues } = this.state;
    const rows = new Map<number, ITemplateAdminMainRow>();
    this.lastMainState.main = deepClone(main);

    main.map(element => {
      const formRow = this.getMainRowFromId(element.id, element.name);
      const newFormRow: ITemplateAdminMainRow = {
        id: formRow.id,
        name: formRow.name,
        values: deepClone(formRow.values)
      };
      rows.set(element.id, newFormRow);
    });

    this.lastMainState.rows = rows;
    this.lastMainState.movableMainValues = this.cloneMovableOptions(movableMainValues);
  };

  private restoreLastMainState = (): void => {
    this.setState({ main: this.lastMainState.main, movableMainValues: this.lastMainState.movableMainValues, hideConfirmMainChange: true });
    this.lastMainState.rows.forEach((row: ITemplateAdminMainRow, id: number) => {
      this.result.main.set(id, row);
    });
  };

  private setPropertiesSingleValue(id: number, fieldName: string, isSystemField: boolean, value: any): void {
    const { properties } = this.state;
    const formRow = this.getPropertiesRowFromId(id, fieldName, isSystemField);
    const fieldValue: IFieldValue = {
      id: null,
      value: value
    };
    formRow.values[0] = fieldValue;
    this.result.properties.set(id, formRow);

    const index = properties.findIndex(p => p.id === id);
    const newProperties = update(properties, { [index]: { values: { $set: [{ text: value }] } } });
    this.setState({ properties: newProperties });
  }

  private handleMainTextInput(event: any, element: any): void {
    const id: number = element.id;
    const fieldName: string = element.name;
    this.setMainSingleValue(id, fieldName, event.target.value);
  }

  private handlePropertiesTextInput(event: any, element: any): void {
    const id: number = element.id;
    const fieldName: string = element.name;
    this.setPropertiesSingleValue(id, fieldName, element.isSystemField, event.target.value);
  }

  private handleMainDateInput(date: Date | null | undefined, element: any): void {
    const id: number = element.id;
    const fieldName: string = element.name;
    if (!date) {
      return;
    }
    this.setMainSingleValue(id, fieldName, date.getTime() / 1000);
  }

  private handlePropertiesDateInput(date: Date | null | undefined, element: any): void {
    const id: number = element.id;
    const fieldName: string = element.name;
    if (!date) {
      return;
    }

    this.setPropertiesSingleValue(id, fieldName, element.isSystemField, date.getTime() / 1000);
  }

  private onMainChangeValues = (event: React.FormEvent<HTMLDivElement>, element: any, item?: IDropdownOption): void => {
    if (!item) {
      return;
    }
    this.canResetMovableResult = true;

    const { main } = this.state;
    const id: number = element.id;
    const fieldName: string = element.name;

    const formRow = this.getMainRowFromId(id, fieldName);
    this.storeLastMainState();

    let updatedSelectedItems: any = [];

    if (item.selected) {
      // add the option if it's checked
      const fieldValue: IFieldValue = {
        id: item.key,
        value: item.text
      };
      updatedSelectedItems = update(formRow.values, { $push: [fieldValue] });
    } else {
      // remove the option if it's unchecked
      const currIndex = formRow.values
        .map(function(x) {
          return x.id;
        })
        .indexOf(item!.key);

      if (currIndex > -1) {
        updatedSelectedItems = update(formRow.values, { $splice: [[currIndex, 1]] });
      }
    }

    formRow.values = updatedSelectedItems;

    this.result.main.set(id, formRow);

    const newElementValues = deepClone(element.values);

    for (let option of newElementValues) {
      if (String(item.text) == String(option.text)) {
        option.selected = item.selected;
      }
    }

    const index = main.findIndex(p => p.id === id);
    const newMain = update(main, { [index]: { values: { $set: newElementValues } } });
    this.setState({ main: newMain }, () => this.openConfirmMainChange());
  };

  private onPropertiesChangeValues = (event: React.FormEvent<HTMLDivElement>, element: any, item?: IDropdownOption): void => {
    const id: number = element.id;
    const fieldName: string = element.name;
    const formRow = this.getPropertiesRowFromId(id, fieldName, element.isSystemField);

    let updatedSelectedItems: any = [];

    if (!item) {
      return;
    }

    if (item.selected) {
      // add the option if it's checked
      const fieldValue: IFieldValue = {
        id: item.key,
        value: item.text
      };
      updatedSelectedItems = update(formRow.values, { $push: [fieldValue] });
    } else {
      // remove the option if it's unchecked
      const currIndex = formRow.values
        .map(function(x) {
          return x.id;
        })
        .indexOf(item.key);
      if (currIndex > -1) {
        updatedSelectedItems = update(formRow.values, { $splice: [[currIndex, 1]] });
      }
    }

    formRow.values = updatedSelectedItems;

    this.result.properties.set(id, formRow);
  };

  private onMainChangeValuesPickup = (event: React.FormEvent<HTMLDivElement>, element: any, item?: IDropdownOption): void => {
    if (!item) {
      return;
    }

    this.canResetMovableResult = true;

    const { main } = this.state;
    const id: number = element.id;
    const fieldName: string = element.name;
    const formRow = this.getMainRowFromId(id, fieldName);
    this.storeLastMainState();

    const fieldValue: IFieldValue = {
      id: item.key,
      value: item.text,
      selected: true
    };
    const updatedSelectedItems = update(formRow.values, { $set: [fieldValue] });
    formRow.values = updatedSelectedItems;

    this.result.main.set(id, formRow);

    if (element.dependentChildId) {
      this.onChangeParentPickupField(event, item, id, element.dependentChildId);
    }
    const newElementValues = deepClone(element.values);

    for (let option of newElementValues) {
      if (String(item.text) === String(option.text)) {
        option.selected = true;
      } else {
        option.selected = false;
      }
    }
    const index = main.findIndex(p => p.id === id);
    const newMain = update(main, { [index]: { values: { $set: newElementValues } } });
    this.setState({ main: newMain }, () => this.openConfirmMainChange());
  };

  private onPropertiesChangeValuesPickup = (event: React.FormEvent<HTMLDivElement>, element: any, item?: IDropdownOption): void => {
    if (!item) {
      return;
    }

    const id: number = element.id;
    const fieldName: string = element.name;
    const formRow = this.getPropertiesRowFromId(id, fieldName, element.isSystemField);

    const fieldValue: IFieldValue = {
      id: item.key,
      value: item.text,
      selected: true
    };

    const updatedSelectedItems = update(formRow.values, { $set: [fieldValue] });
    formRow.values = updatedSelectedItems;

    this.result.properties.set(id, formRow);
  };

  private onChangePropertyType = (event: React.FormEvent<HTMLDivElement>, item: IDropdownOption | undefined, element: any): void => {
    if (!item) return;

    const id: any = element.id;
    const fieldName: string = element.name;
    const { properties, settingsOperatorsPropertyType } = this.state;
    const index = properties.findIndex(p => String(p.id) === String(id));
    const itemKey = item.key;
    const formRow = this.getPropertiesRowFromId(id, fieldName, element.isSystemField);

    formRow.propertyType = itemKey;

    this.result.properties.set(id, formRow);

    const newProperties = update(properties, { [index]: { type: { $set: itemKey } } });

    const newSettingsOperatorsPropertyType = cloneMap(settingsOperatorsPropertyType);
    newSettingsOperatorsPropertyType.set(id, itemKey);

    this.setState({ properties: newProperties, settingsOperatorsPropertyType: newSettingsOperatorsPropertyType });
  };

  private getMainRowFromId = (id: number, name: string): ITemplateAdminMainRow => {
    let formRow = this.result.main.get(id);
    if (!formRow) {
      formRow = {
        id: id,
        name: name,
        values: []
      };
    }
    return formRow;
  };

  private getPropertiesRowFromId = (id: number, name: string, isSystemField: boolean): ITemplateAdminPropertiesRow => {
    let formRow = this.result.properties.get(id);
    if (!formRow) {
      formRow = {
        id: id,
        name: name,
        isSystemField: isSystemField,
        selectedOperation: -1,
        values: [],
        operations: [],
        propertyType: -1
      };
    }
    return formRow;
  };

  private resetState = (): void => {
    const { id } = this.state;
    const newMain = deepClone(this.initialState.main);
    const newProperties = deepClone(this.initialState.properties);
    const newDisplayProperties = deepClone(this.initialState.displayProperties);
    const newFieldProperties = deepClone(this.initialState.fieldProperties);
    const newOperatorsProperties = cloneMap(this.initialState.operatorsProperties);

    this.setState(
      {
        name: this.initialState.name,
        main: newMain,
        properties: newProperties,
        displayProperties: newDisplayProperties,
        hideDeleteDialogs: [],
        hideResetDialog: true,
        fieldProperties: newFieldProperties,
        operatorsProperties: newOperatorsProperties,
        movableMainValues: new Map<number, IMovableOptionsListElement[]>()
      },
      () => {
        this.resetResultMainValues(id, this.initialState.name, newMain);
        this.setResultPropertiesValues(newProperties);
        this.setDisplayProperties(newDisplayProperties);
      }
    );
  };

  private resetResultMainValues = (id: any, name: string, main: any[]): void => {
    this.result.id = id;
    this.result.name = name;
    this.result.main.clear();

    for (let item of main) {
      const row: ITemplateAdminMainRow = {
        id: item.id,
        name: item.name,
        values: this.getItemValues(item)
      };
      this.result.main.set(item.id, row);
    }
  };

  private setDisplayProperties = (display: any[]): void => {
    const newProperties = [];
    this.result.displayProperties.splice(0, this.result.displayProperties.length);
    for (let property of display) {
      const cloneProperty = deepClone(property);
      newProperties.push(cloneProperty);
      if (property.selected) {
        this.result.displayProperties.push(property.key);
      }
    }
    this.setState({
      displayProperties: newProperties
    });
  };

  private setResultPropertiesValues = (properties: any[]): void => {
    const oldProperties = cloneMap(this.result.properties);
    this.result.properties.clear();

    const newSettingsOperators = new Map<number, boolean[]>();
    const newSettingsOperatorsSelectionType: string[] = [];
    const newSettingsOperatorsPropertyType = new Map<number, number>();

    for (let item of properties) {
      let operCount = 0;
      const settingsValues: boolean[] = [];
      const newOperations: any[] = [];
      for (let operation of item.operations) {
        settingsValues[operation.key] = true;
        const value = {
          key: operation.key,
          text: operation.text,
          selected: operation.selected
        };
        newOperations.push(value);
        operCount++;
      }

      const elementOperatorProperties: IDropdownOption[] = this.getElementOperatorProperties(item.id);

      newSettingsOperatorsSelectionType[item.id] =
        operCount === elementOperatorProperties.length ? OperatorsSettingsTypeEnum.USE_ALL : OperatorsSettingsTypeEnum.USE_SELECTED;

      newSettingsOperators.set(item.id, settingsValues);

      newSettingsOperatorsPropertyType.set(item.id, item.type);

      const oldPropertiesRow: ITemplateAdminPropertiesRow | undefined = oldProperties.get(item.id);
      const oldPropertyValues: IFieldValue[] | undefined = oldPropertiesRow ? oldPropertiesRow.values : undefined;

      const row: ITemplateAdminPropertiesRow = {
        id: item.id,
        name: item.name,
        isSystemField: item.isSystemField,
        selectedOperation: this.getOperationValues(item),
        values: this.getItemValues(item, oldPropertyValues),
        operations: newOperations,
        propertyType: item.type
      };
      this.result.properties.set(item.id, row);
    }

    this.setState({
      settingsOperators: newSettingsOperators,
      settingsOperatorsSelectionType: newSettingsOperatorsSelectionType,
      settingsOperatorsPropertyType: newSettingsOperatorsPropertyType
    });
  };

  private getItemValues = (item: any, oldValues: IFieldValue[] | undefined = undefined): any[] => {
    const result: any[] = [];

    if (item.values == null) {
      return result;
    }

    const singleField = item.field !== FieldTypeEnum.LOOKUP && item.field !== FieldTypeEnum.PICKUP && item.field !== FieldTypeEnum.MOVABLE;

    for (let obj of item.values) {
      const olderValue = oldValues ? oldValues.find(i => i.id === obj.key) : undefined;
      const fieldValue: IFieldValue = {
        id: obj.key,
        value: olderValue ? olderValue.value : obj.text
      };
      if (!singleField) {
        if (obj.selected || olderValue) {
          result.push(fieldValue);
        }
      } else {
        result.push(fieldValue);
      }
    }

    return result;
  };

  private getOperationValues = (item: any): any => {
    let operation = '';
    for (let obj of item.operations) {
      if (obj.selected) {
        operation = obj.key;
        break;
      }
    }

    return operation;
  };

  private showDeleteDialog = (id: any): void => {
    const { hideDeleteDialogs } = this.state;
    const newHideDeleteDialogs = update(hideDeleteDialogs, { [id]: { $set: false } });
    this.setState({ hideDeleteDialogs: newHideDeleteDialogs });
  };

  private closeDeleteDialog = (id: any): void => {
    const { hideDeleteDialogs } = this.state;
    const newHideDeleteDialogs = update(hideDeleteDialogs, { [id]: { $set: true } });
    this.setState({ hideDeleteDialogs: newHideDeleteDialogs });
  };

  private showResetDialog = (): void => {
    this.setState({ hideResetDialog: false });
  };

  private closeResetDialog = (): void => {
    this.setState({ hideResetDialog: true });
  };

  private onChangeParentPickupField = (
    event: React.FormEvent<HTMLDivElement>,
    item: IDropdownOption,
    id: number,
    dependentId: number
  ): void => {
    const values: any[] = [];
    values.push(item.key);
    this.onChangeParentField(id, values, dependentId);
  };

  private onChangeParentField = (parentId: number, values: any[], dependentId: number): void => {
    let idsUriValues = '';

    for (let id of values) {
      idsUriValues = idsUriValues + `&ids=${id}`;
    }

    const url = `${this.url}/main/dependent/${parentId}/${dependentId}?${idsUriValues}`;

    console.log(url);

    Axios.get(url)
      .then(response => {
        this.updateChildMainField(response.data, dependentId);
      })
      .catch(error => console.log(error));
  };

  private updateChildMainField = (response: any, dependentId: number): void => {
    const { movableMainValues } = this.state;
    const newMovableMainValues = cloneMap(movableMainValues);

    newMovableMainValues.set(dependentId, response.data);
    this.setState({ movableMainValues: newMovableMainValues });
  };

  private getOperatorActionItems = (element: any, idx: number, length: number) => {
    const { labelIconOperatorDelete, labelIconOperatorSetting, labelIconOperatorUp, labelIconOperatorDown } = this.props;
    const isFirst: boolean = idx == 0;
    const isLast: boolean = idx == length - 1;
    const upIndex = 2;
    const downIndex = 3;

    const items = [
      {
        key: 'Delete',
        name: labelIconOperatorDelete,
        iconProps: {
          iconName: 'DeleteRows'
        },
        iconOnly: true,
        onClick: () => this.showDeleteDialog(element.id),
        style: { backgroundColor: '#fff' }
      },
      {
        key: 'Settings',
        name: labelIconOperatorSetting,
        iconProps: {
          iconName: 'Settings'
        },
        iconOnly: true,
        onClick: () => this.onShowSettingsPanel(element),
        style: { backgroundColor: '#fff' }
      },
      {
        key: 'Up',
        name: labelIconOperatorUp,
        iconProps: {
          iconName: 'Up'
        },
        iconOnly: true,
        onClick: () => this.movePropertyUp(element.id),
        style: { backgroundColor: '#fff' }
      },
      {
        key: 'Down',
        name: labelIconOperatorDown,
        iconProps: {
          iconName: 'Down'
        },
        iconOnly: true,
        onClick: () => this.movePropertyDown(element.id),
        style: { backgroundColor: '#fff' }
      }
    ];

    if (isFirst || isLast) {
      const removeIndex = isFirst ? upIndex : downIndex;
      items.splice(removeIndex, 1);
    }

    return items;
  };

  private getFormCommandBarItems = (id: any) => {
    const { labelCmdBarBtnSave, labelCmdBarBtnEdit, labelCmdBarBtnCancel } = this.props;

    const formCommandBarItems = [
      {
        key: 'Save',
        name: labelCmdBarBtnSave,
        iconProps: {
          iconName: 'Save'
        },
        onClick: () => this.onSubmit()
      },
      {
        key: 'Edit',
        name: labelCmdBarBtnEdit,
        iconProps: {
          iconName: 'Edit'
        },
        onClick: () => this.onSubmit()
      },
      {
        key: 'Cancel',
        name: labelCmdBarBtnCancel,
        iconProps: {
          iconName: 'Cancel'
        },
        onClick: () => this.props.onCancel()
      }
    ];

    const removeIdx = id ? 0 : 1;
    formCommandBarItems.splice(removeIdx, 1);

    return formCommandBarItems;
  };

  private getFormCommandBarFarItems = () => {
    const { labelCmdBarFarBtnAdd, labelCmdBarFarBtnReset, labelCmdBarFarBtnResults, allowDisplayProperties, allowMenus } = this.props;
    const displaycommandsIndex = 2;
    const menuCommandsIndex = 3;

    const commands = [
      {
        key: 'Add Property',
        name: labelCmdBarFarBtnAdd,
        iconProps: {
          iconName: 'Add'
        },
        onClick: () => this.onShowAddPropertiesPanel(true)
      },
      {
        key: 'Reset',
        name: labelCmdBarFarBtnReset,
        iconProps: {
          iconName: 'Refresh'
        },
        onClick: () => this.showResetDialog()
      },
      {
        key: 'Display Results',
        name: labelCmdBarFarBtnResults,
        iconProps: {
          iconName: 'ShowResults'
        },
        onClick: () => this.onShowDisplayPropertiesPanel()
      },
      {
        key: 'Add Menu',
        name: 'Add Menu',
        iconProps: {
          iconName: 'CollapseMenu'
        },
        onClick: () => this.onShowMenusPanel(true)
      }
    ];

    if (!allowDisplayProperties) {
      commands.splice(displaycommandsIndex, 1);
    }

    if (!allowMenus) {
      commands.splice(menuCommandsIndex, 1);
    }

    return commands;
  };

  private onBuildPropertiesColumns = () => {
    return [
      {
        key: 'Property',
        name: 'Property',
        fieldName: 'text',
        minWidth: 50,
        maxWidth: 70,
        isRowHeader: true,
        isResizable: false,
        isSorted: true,
        isSortedDescending: false,
        sortAscendingAriaLabel: 'Sorted A to Z',
        sortDescendingAriaLabel: 'Sorted Z to A',
        data: 'string',
        isPadded: true
      }
    ];
  };

  private onBuildMenusColumns = () => {
    return [
      {
        key: 'Name',
        name: 'Menu',
        fieldName: 'name',
        minWidth: 50,
        maxWidth: 100,
        isRowHeader: true,
        isResizable: false,
        isSorted: true,
        isSortedDescending: false,
        sortAscendingAriaLabel: 'Sorted A to Z',
        sortDescendingAriaLabel: 'Sorted Z to A',
        data: 'string',
        isPadded: true
      },
      {
        key: 'Description',
        name: 'Description',
        fieldName: 'description',
        minWidth: 150,
        maxWidth: 200,
        isRowHeader: true,
        isResizable: false,
        isSorted: true,
        isSortedDescending: false,
        sortAscendingAriaLabel: 'Sorted A to Z',
        sortDescendingAriaLabel: 'Sorted Z to A',
        data: 'string',
        isPadded: true
      }
    ];
  };

  private getFieldTypeForProperty = (id: number): any => {
    const index = this.state.fieldProperties.findIndex(p => Number(p.key) === Number(id));
    const item = this.state.fieldProperties[index];
    return item
      ? {
          id: item.key,
          value: item.text,
          selected: true
        }
      : {};
  };

  private getOperationsForProperty = (id: number, operations: any[]): any[] => {
    const propertyOperations: any[] = [];

    operations.forEach((obj: any, key: number) => {
      const operId = Number(obj.key);
      const operation = {
        id: operId,
        value: obj.text,
        selected: operId == id ? true : false
      };
      propertyOperations.push(operation);
    });

    return propertyOperations;
  };

  private getDynamicTranslationInternal = (key: string): string => {
    const { getDynamicTranslation } = this.props;
    return getDynamicTranslation ? getDynamicTranslation(key) : key;
  };

  private openConfirmMainChange = (): void => {
    const { properties, movableMainValues } = this.state;
    let hasMovableValues = false;

    movableMainValues.forEach((values: IMovableOptionsListElement[], key: number) => {
      for (let value of values) {
        if (value.selected) {
          hasMovableValues = true;
          break;
        }
      }
    });

    const hasPropertiesElements = properties.length > 0 || !this.checkIfCanOpenDisplayProperties() || hasMovableValues;

    if (hasPropertiesElements) {
      this.setState({ hideConfirmMainChange: false });
    }
  };

  private validate = (): boolean => {
    this.setState({ validateMainFieldsError: undefined, validatePropertiesFieldsError: false, validateDisplayPropertiesError: false });
    const isValidMainFields = this.validateMainFields();
    const isValidPropertiesFields = this.validatePropertiesFields();
    const isValidDisplayProperties = this.validateDisplayProperties();
    return isValidMainFields && isValidPropertiesFields && isValidDisplayProperties;
  };

  private validatePropertiesFields = (): boolean => {
    const isValid = this.result.properties.size > 0;

    if (!isValid) {
      this.setState({ validatePropertiesFieldsError: true });
      this.delayedExecution(this.closePropertiesFieldsMessage);
    }
    return isValid;
  };

  private validateDisplayProperties = (): boolean => {
    const { allowDisplayProperties } = this.props;
    if (!allowDisplayProperties) {
      return true;
    }
    const isValid = this.result.displayProperties.length > 0;

    if (!isValid) {
      this.setState({ validateDisplayPropertiesError: true });
      this.delayedExecution(this.closeDisplayPropertiesMessage);
    }
    return isValid;
  };

  private validateMainFields = (checkInput: boolean = true): boolean => {
    let invalid: string[] = [];

    if (this.result === null) {
      return false;
    }

    const { main } = this.state;
    let isValid = true;

    this.result.main.forEach((row: ITemplateAdminMainRow, key: number) => {
      const index = main.findIndex(p => p.id === row.id);
      const element = main[index];

      const isInputFied = element.field === FieldTypeEnum.INPUT || element.field === FieldTypeEnum.DATEPICKER;
      const needToCheckInput = checkInput && isInputFied;

      if (element.required) {
        if (!isInputFied || needToCheckInput) {
          const isValid = row.values && row.values.length > 0;
          if (!isValid) {
            invalid.push(row.name);
          }
        }
      }
    });

    if (invalid.length > 0) {
      const errorsAsString = invalid.map((error: string) => error).join(' | ');
      this.setState({ validateMainFieldsError: `[ ${errorsAsString} ]` });
      isValid = false;
      this.delayedExecution(this.closeMainFieldsMessage);
    }

    return isValid;
  };

  private onSubmit = (): void => {
    const { id, onSubmitSuccess, onSubmitError } = this.props;

    if (!this.validate()) {
      return;
    }

    let main: ITemplateAdminMainRow[] = [];
    let properties: ITemplateAdminPropertiesRow[] = [];

    this.result.main.forEach((row: ITemplateAdminMainRow, key: number) => {
      main.push(row);
    });

    this.result.properties.forEach((row: ITemplateAdminPropertiesRow, key: number) => {
      row.propertyType = this.getFieldTypeForProperty(row.propertyType);
      row.operations = this.getOperationsForProperty(row.selectedOperation, row.operations);
      properties.push(row);
    });

    this.logResult(this.result);

    const jsonResult = JSON.stringify({
      id: this.result.id,
      name: this.result.name,
      main: main,
      properties: properties,
      displayProperties: this.result.displayProperties,
      menuId: this.result.menuId
    });

    const method = id ? 'post' : 'put';
    const url = id ? `${this.url}/${id}` : this.url;

    console.log('Data Submited to server by', method, jsonResult);

    Axios.request({
      url: url,
      method: method,
      data: jsonResult,
      headers: {
        Accept: 'application/json',
        'Content-Type': 'application/json'
      }
    })
      .then(response => {
        onSubmitSuccess(response.data);
      })
      .catch(error => onSubmitError(error));
  };

  private getSelectedKeys = (values: any[]): any => {
    let keys = [];

    for (let option of values) {
      if (option.selected) {
        keys.push(option.key);
      }
    }
    return keys;
  };

  private getSelectedKey = (values: any[]): any => {
    let key = 0;

    for (let option of values) {
      if (option.selected) {
        key = option.key;
        break;
      }
    }
    return key;
  };

  private delayedExecution = (callback: any): void => {
    setTimeout(callback, this.delayedTimeout);
  };

  private closeMainFieldsMessage = (): void => {
    this.setState({ validateMainFieldsError: undefined });
  };

  private closePropertiesFieldsMessage = (): void => {
    this.setState({ validatePropertiesFieldsError: false });
  };

  private closeDisplayPropertiesMessage = (): void => {
    this.setState({ validateDisplayPropertiesError: false });
  };

  private logResult = (result: ITemplateAdminFormObject): void => {
    if (result === null) {
      console.log('result empty');
      return;
    }

    let msg = '\n\n------------------------\n';
    msg += '      Form Values       \n';
    msg += '------------------------';
    msg += '\nId:' + result.id;
    msg += '\nName:' + result.name;
    var displayMsg = '\nDisplay Properties:\n';
    result.displayProperties.forEach(element => {
      displayMsg += `\tkey=${element.key}; name=${element.text};isSystemField=${element.isSystemField}\n`;
    });
    msg += displayMsg;
    msg += '\nMenu ID: ' + result.menuId;
    msg += '\n------------------------\n';
    msg += '\n Main Values';
    msg += '\n------------------------';

    result.main.forEach((row: ITemplateAdminMainRow, key: number) => {
      msg += '\n   ID: ' + row.id + '\n';
      msg += '\n   Name: ' + row.name + '\n';

      let valuesMsg = '   Values:\n';
      row.values.forEach((field: IFieldValue, key: number) => {
        valuesMsg += `\t${field.id};${field.value}\n`;
      });
      msg += valuesMsg;
      msg += '------------------------';
    });

    msg += '\n------------------------\n';
    msg += '\n Properties Values';
    msg += '\n------------------------';

    result.properties.forEach((row: ITemplateAdminPropertiesRow, key: number) => {
      msg += '\n   ID: ' + row.id + '\n';
      msg += '\n   Name: ' + row.name + '\n';
      msg += '\n   System Field: ' + row.isSystemField + '\n';

      let valuesMsg = '   Values:\n';
      row.values.forEach((field: IFieldValue, key: number) => {
        valuesMsg += `\t${field.id};${field.value}\n`;
      });
      msg += valuesMsg;
      msg += '------------------------\n\n';

      let settingsMsg = '   Operations:\n';
      settingsMsg += `\tProperty Type: ${row.propertyType.id}:${row.propertyType.value}\n`;

      row.operations.forEach((operation: any, key: number) => {
        settingsMsg += `\tid: ${operation.id} => ${operation.value} : ${operation.selected}\n`;
      });
      msg += settingsMsg;
      msg += '------------------------';
    });

    msg += '\n\n';
    console.log(msg);
  };
}
