import * as React from 'react';

import { Fabric } from 'office-ui-fabric-react/lib/Fabric';

import { classNamesFunction } from 'office-ui-fabric-react/lib/Utilities';

import { TextField } from 'office-ui-fabric-react/lib/TextField';

import { Dropdown, IDropdownOption } from 'office-ui-fabric-react/lib/Dropdown';

import { DatePicker } from 'office-ui-fabric-react/lib/DatePicker';

import { Label } from 'office-ui-fabric-react/lib/Label';

import { Title } from '../Title';

import { CommandBar } from 'office-ui-fabric-react/lib/CommandBar';

import { Dialog, DialogType, DialogFooter } from 'office-ui-fabric-react/lib/Dialog';

import { ActionButton, DefaultButton, PrimaryButton } from 'office-ui-fabric-react/lib/Button';

import { Panel, PanelType } from 'office-ui-fabric-react/lib/Panel';

import { FieldTypeEnum } from '../../Utils/Utils.types';

import { formToTemplateResult } from '../../Utils/TemplateUtils';

import update from 'immutability-helper';

import './TemplateForm.css';

import {
  ITemplateFormProps,
  ITemplateFormStyleProps,
  ITemplateFormStyles,
  ITemplateFormRow,
  ITemplateFormSearchObject,
  RenderType,
  ITemplateDto
} from './TemplateForm.types';
import Axios from 'axios';

const getClassNames = classNamesFunction<ITemplateFormStyleProps, ITemplateFormStyles>();

export interface ITemplateFormState {
  id: number;
  name: string;
  properties: any[];
  hideExternalSurface: boolean;
  error: any;
}

export class TemplateFormBase extends React.PureComponent<ITemplateFormProps, ITemplateFormState> {
  public static defaultProps = {
    renderType: RenderType.COMMANDBAR,
    labelCmdBarSearch: 'Search',
    labelCmdBarCancel: 'Cancel'
  };
  private result: ITemplateFormSearchObject = { values: new Map<number, ITemplateFormRow>(), displayColumns: [], main: [] };
  constructor(props: ITemplateFormProps) {
    super(props);
    this.state = {
      id: -1,
      name: '',
      properties: [],
      hideExternalSurface: this.props.hideExternalSurface != null ? this.props.hideExternalSurface : true,
      error: undefined
    };
    this.onSubmit = this.onSubmit.bind(this);
    this.onCancel = this.onCancel.bind(this);
    this.getRowFromId = this.getRowFromId.bind(this);
    this.setSingleValue = this.setSingleValue.bind(this);
  }

  public componentDidMount() {
    Axios.get(this.props.url)
      .then(response => {
        const dto: ITemplateDto = response.data;
        this.setState({
          id: dto.id,
          name: dto.name,
          properties: dto.data
        });
        this.resetResultValues(dto.data);
        this.result.displayColumns = dto.displayColumns;
        this.result.main = dto.main;
      })
      .catch(error => this.setState({ error: error }));
  }

  private renderForm(): JSX.Element {
    const { className, styles, theme } = this.props;
    const { name, properties } = this.state;
    const { renderType } = this.props;
    const classNames = getClassNames(styles, { theme: theme!, className, renderType });

    return (
      <div>
        {renderType === RenderType.COMMANDBAR && (
          <div className={`ms-Grid-row ${classNames.commandBar}`}>
            <div className="ms-Grid-col ms-sm12 ms-md12 ms-lg12">
              <CommandBar items={this.getCommandBarItems()} />
            </div>
          </div>
        )}
        <div className="ms-Grid-row tpl-f-nl-sm">
          <div className="ms-Grid-col ms-sm12 ms-md12 ms-lg12" />
        </div>
        {renderType !== RenderType.DIALOG && (
          <div className={`ms-Grid-row ${classNames.title}`}>
            <div className="ms-Grid-col ms-sm12 ms-md12 ms-lg12">
              <Title title={name} icon={true} />
            </div>
          </div>
        )}
        <div className={`${classNames.form}`}>
          {properties.map(element => {
            let field = undefined;
            const rowKey = `Row_${element.id}`;
            const fieldName = element.name;
            const operationsKey = `DropdownOperations_${element.id}`;

            switch (element.field) {
              case FieldTypeEnum.INPUT: {
                const key = `TextField_${element.id}`;
                field = (
                  <TextField
                    key={key}
                    value={this.getSafeValue(element.values)}
                    required={element.required}
                    onBlur={e => this.handleTextInput(e, element.id, element.name, element.isSystemField)}
                  />
                );
                break;
              }
              case FieldTypeEnum.LOOKUP: {
                const key = `Dropdown_${element.id}`;
                field = (
                  <Dropdown
                    multiSelect
                    key={key}
                    options={element.values}
                    onChange={(event: React.FormEvent<HTMLDivElement>, option?: IDropdownOption | undefined, index?: number | undefined) =>
                      this.onChangeValues(event, element.id, element.name, element.isSystemField, option)
                    }
                    required={element.required}
                  />
                );
                break;
              }
              case FieldTypeEnum.PICKUP: {
                const key = `Pickup_${element.id}`;
                field = (
                  <Dropdown
                    key={key}
                    options={element.values}
                    onChange={(event: React.FormEvent<HTMLDivElement>, option?: IDropdownOption | undefined, index?: number | undefined) =>
                      this.onChangeValuesPickup(event, element.id, element.name, element.isSystemField, option)
                    }
                    required={element.required}
                  />
                );
                break;
              }
              case FieldTypeEnum.DATEPICKER: {
                const key = `DatePicker_${element.id}`;
                field = (
                  <DatePicker
                    key={key}
                    onSelectDate={(d: Date | null | undefined) => this.handleDateInput(d, element.id, element.name, element.isSystemField)}
                    value={this.getSafeDateValue(element.values)}
                  />
                );
                break;
              }
              default: {
                break;
              }
            }
            return (
              <div className="ms-Grid-row" key={rowKey}>
                <div className="ms-Grid-col ms-sm3 ms-md3 ms-lg3">
                  <Label className="ms-font-m-plus">{fieldName}:</Label>
                </div>
                <div className="ms-Grid-col ms-sm4 ms-md4 ms-lg4">
                  <Dropdown
                    key={operationsKey}
                    options={element.operations}
                    onChange={(event: React.FormEvent<HTMLDivElement>, option?: IDropdownOption | undefined, index?: number | undefined) =>
                      this.onChangeOperation(event, element.id, element.name, element.isSystemField, option)
                    }
                  />
                </div>
                <div className="ms-Grid-col ms-sm5 ms-md5 ms-lg5">{field}</div>
                <div className="ms-Grid-row tpl-f-nl-md">
                  <div className="ms-Grid-col ms-sm12 ms-md12 ms-lg12" />
                </div>
              </div>
            );
          })}
        </div>
      </div>
    );
  }

  public render(): JSX.Element {
    const { className, styles, theme, renderType, labelCmdBarSearch, labelCmdBarCancel } = this.props;
    const { name, hideExternalSurface } = this.state;
    const classNames = getClassNames(styles, { theme: theme!, className, renderType });

    return (
      <Fabric className={`ms-Grid ${classNames.root}`}>
        {renderType === RenderType.DIALOG && !hideExternalSurface && (
          <Dialog
            hidden={hideExternalSurface}
            minWidth={'650px'}
            title={name}
            onDismiss={this.closeExternalSurface}
            dialogContentProps={{
              type: DialogType.normal
            }}
            modalProps={{
              isDarkOverlay: true,
              isBlocking: true,
              containerClassName: 'ms-dialogMainOverride'
            }}
          >
            {this.renderForm()}
            <DialogFooter>
              <PrimaryButton onClick={this.onSubmit} text={labelCmdBarSearch} />
              <DefaultButton onClick={this.onCancel} text={labelCmdBarCancel} />
            </DialogFooter>
          </Dialog>
        )}
        {renderType === RenderType.PANEL && !hideExternalSurface && (
          <Panel
            isOpen={!hideExternalSurface}
            type={PanelType.custom}
            customWidth="650px"
            onDismiss={this.closeExternalSurface}
            closeButtonAriaLabel="Close"
          >
            {this.renderForm()}
            <div className="ms-Grid">
              <div className="ms-Grid-row tpl-f-nl-md">
                <div className="ms-Grid-col ms-sm12 ms-md12 ms-lg12" />
              </div>
              <div className="ms-Grid-row">
                <div className="ms-Grid-col ms-sm8 ms-md8 ms-lg8" />
                <div className="ms-Grid-col ms-sm2 ms-md2 ms-lg2">
                  <ActionButton iconProps={{ iconName: 'Search' }} onClick={this.onSubmit}>
                    {labelCmdBarSearch}
                  </ActionButton>
                </div>
                <div className="ms-Grid-col ms-sm2 ms-md2 ms-lg2">
                  <ActionButton iconProps={{ iconName: 'Cancel' }} onClick={this.onCancel}>
                    {labelCmdBarCancel}
                  </ActionButton>
                </div>
              </div>
            </div>
          </Panel>
        )}
        {renderType === RenderType.COMMANDBAR && this.renderForm()}
      </Fabric>
    );
  }

  private closeExternalSurface = (): void => {
    this.setState({ hideExternalSurface: true });
  };

  private getSafeValue = (values: any[]): any => {
    return values.length ? values[0].text : '';
  };

  private getSafeDateValue = (values: any[]): any => {
    return values.length ? this.getSafeTextToDate(values[0].text) : null;
  };

  private getSafeTextToDate = (value: any): any => {
    let safeDate = null;
    try {
      const asNumber: number = Number(value);
      safeDate = new Date(0);
      safeDate.setUTCSeconds(asNumber);
    } catch (e) {
      console.log('Error converting text to date:', value);
    }

    return safeDate;
  };

  private setSingleValue(id: number, name: string, isSystemField: boolean, value: any): void {
    const formRow = this.getRowFromId(id, name, isSystemField);
    const fieldValue = {
      key: id,
      text: name
    };
    formRow.values[0] = fieldValue;
    this.result.values.set(id, formRow);
  }

  private handleTextInput(event: any, id: number, name: string, isSystemField: boolean): void {
    this.setSingleValue(id, name, isSystemField, event.target.value);
  }

  private handleDateInput(date: Date | null | undefined, id: number, name: string, isSystemField: boolean): void {
    this.setSingleValue(id, name, isSystemField, date);
  }

  private onChangeOperation = (
    event: React.FormEvent<HTMLDivElement>,
    id: number,
    name: string,
    isSystemField: boolean,
    option?: IDropdownOption | undefined
  ): void => {
    if (!option) {
      return;
    }

    const formRow = this.getRowFromId(id, name, isSystemField);
    formRow.operation = Number(option.key);

    this.result.values.set(id, formRow);
  };

  private onChangeValues = (
    event: React.FormEvent<HTMLDivElement>,
    id: number,
    name: string,
    isSystemField: boolean,
    option?: IDropdownOption | undefined
  ): void => {
    if (!option) {
      return;
    }
    const formRow = this.getRowFromId(id, name, isSystemField);

    let updatedSelectedItems: any = [];

    if (option.selected) {
      // add the option if it's checked
      const fieldValue = {
        key: option.key,
        text: option.text
      };
      updatedSelectedItems = update(formRow.values, { $push: [fieldValue] });
    } else {
      // remove the option if it's unchecked
      const currIndex = formRow.values
        .map(function(x) {
          return x.id;
        })
        .indexOf(option.key);
      if (currIndex > -1) {
        updatedSelectedItems = update(formRow.values, { $splice: [[currIndex, 1]] });
      }
    }

    formRow.values = updatedSelectedItems;

    this.result.values.set(id, formRow);
  };

  private onChangeValuesPickup = (
    event: React.FormEvent<HTMLDivElement>,
    id: number,
    name: string,
    isSystemField: boolean,
    option?: IDropdownOption | undefined
  ): void => {
    if (!option) {
      return;
    }

    const formRow = this.getRowFromId(id, name, isSystemField);

    const fieldValue = {
      key: option.key,
      text: option.text
    };
    const updatedSelectedItems = update(formRow.values, { $push: [fieldValue] });
    formRow.values = updatedSelectedItems;

    this.result.values.set(id, formRow);
  };

  private getRowFromId = (id: number, name: string, isSystemField: boolean): ITemplateFormRow => {
    let formRow = this.result.values.get(id);
    if (!formRow) {
      formRow = {
        id: id,
        name: name,
        isSystemField: isSystemField,
        operation: -1,
        values: []
      };
    }
    return formRow;
  };

  private getCommandBarItems = () => {
    const { labelCmdBarSearch, labelCmdBarCancel } = this.props;
    const formCommandBarItems = [
      {
        key: 'Search',
        name: labelCmdBarSearch,
        iconProps: {
          iconName: 'Search'
        },
        onClick: () => this.onSubmit()
      },
      {
        key: 'Cancel',
        name: labelCmdBarCancel,
        iconProps: {
          iconName: 'Cancel'
        },
        onClick: () => this.props.onCancel()
      }
    ];
    return formCommandBarItems;
  };

  private resetResultValues = (data: any[]): void => {
    this.result.values.clear();

    for (let item of data) {
      const row: ITemplateFormRow = {
        id: item.id,
        name: item.name,
        isSystemField: item.isSystemField,
        operation: this.getOperationsValue(item.operations),
        values: this.getValues(item.values)
      };
      this.result.values.set(item.id, row);
    }
  };

  private getOperationsValue = (operations: any[]): any => {
    let result = -1;
    for (let item of operations) {
      if (item.selected) {
        result = item.key;
        break;
      }
    }
    return result;
  };

  private getValues = (values: any[]): any => {
    let result: any[] = [];
    for (let item of values) {
      const fieldValue = {
        key: item.key,
        text: item.text
      };
      if (item.selected === null) {
        result.push(fieldValue);
      } else {
        if (item.selected) {
          result.push(fieldValue);
        }
      }
    }
    return result;
  };

  private onCancel = (): void => {
    this.setState({ hideExternalSurface: true }, () => this.props.onCancel());
  };

  private onSubmit = (): void => {
    const { onSubmit } = this.props;
    const encoded = formToTemplateResult(this.state, this.result);

    this.setState({ hideExternalSurface: true }, () => onSubmit(encoded));
  };
}
